import os
import unittest
import json
from typing import Dict
import jc.parsers.sshd_conf

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):
    f_in: Dict = {}
    f_json: Dict = {}

    @classmethod
    def setUpClass(cls):
        fixtures = {
            'sshd_t': (
                'fixtures/generic/sshd-T.out',
                'fixtures/generic/sshd-T.json'),
            'sshd_t_2': (
                'fixtures/generic/sshd-T-2.out',
                'fixtures/generic/sshd-T-2.json'),
            'sshd_config': (
                'fixtures/generic/sshd_config',
                'fixtures/generic/sshd_config.json')
        }

        for file, filepaths in fixtures.items():
            with open(os.path.join(THIS_DIR, filepaths[0]), 'r', encoding='utf-8') as a, \
                 open(os.path.join(THIS_DIR, filepaths[1]), 'r', encoding='utf-8') as b:
                cls.f_in[file] = a.read()
                cls.f_json[file] = json.loads(b.read())


    def test_sshd_conf_nodata(self):
        """
        Test 'sshd_conf' with no data
        """
        self.assertEqual(jc.parsers.sshd_conf.parse('', quiet=True), {})

    def test_sshd_T(self):
        """
        Test 'sshd -T'
        """
        self.assertEqual(jc.parsers.sshd_conf.parse(self.f_in['sshd_t'], quiet=True),
                                                    self.f_json['sshd_t'])

    def test_sshd_T_2(self):
        """
        Test 'sshd -T' with another sample
        """
        self.assertEqual(jc.parsers.sshd_conf.parse(self.f_in['sshd_t_2'], quiet=True),
                                                    self.f_json['sshd_t_2'])

    def test_sshd_config(self):
        """
        Test 'cat sshd_config'
        """
        self.assertEqual(jc.parsers.sshd_conf.parse(self.f_in['sshd_config'], quiet=True),
                                                    self.f_json['sshd_config'])


if __name__ == '__main__':
    unittest.main()



r"""jc - JSON Convert `apt-cache show` command parser

Usage (cli):

    $ apt-cache show | jc --apt-cache-show

Usage (module):

    import jc
    result = jc.parse('apt_cache_show', apt_cache_show_output)

Schema:

    [
      {
        "package":                string,
        "version":                string,
        "installed_size":         integer,
        "maintainer":             string,
        "architecture":           string,
        "depends": [
                                  string
        ],
        "pre_depends": [
                                  string
        ],
        "recommends": [
                                  string
        ],
        "suggests": [
                                  string
        ],
        "conflicts": [
                                  string
        ],
        "breaks": [
                                  string
        ],
        "description_md5":        string,
        "multi_arch":             string,
        "homepage":               string,
        "section":                string,
        "priority":               string,
        "filename":               string,
        "size":                   integer,
        "sha256":                 string,
        "description":            string
      }
    ]

Examples:

    $ apt-cache show | jc --apt-cache-show -p
    [
      {
        "package": "systemd",
        "version": "247.3-6~bpo10+1",
        "installed_size": 16121,
        "maintainer": "Debian systemd Maintainers <pkg-systemd-maintaine..",
        "architecture": "amd64",
        "depends": [
          "libacl1 (>= 2.2.23)",
          "libapparmor1 (>= 2.13)",
          "libaudit1 (>= 1:2.2.1)",
          "libcap2 (>= 1:2.10)",
          "libcryptsetup12 (>= 2:2.0.1)",
          "libgnutls30 (>= 3.6.6)",
          "libgpg-error0 (>= 1.14)",
          "libip4tc0 (>= 1.6.0+snapshot20161117)",
          "libkmod2 (>= 5~)",
          "liblz4-1 (>= 0.0~r130)",
          "libmount1 (>= 2.30)",
          "libpam0g (>= 0.99.7.1)",
          "libseccomp2 (>= 2.3.1)",
          "libsystemd0 (= 247.3-6~bpo10+1)",
          "systemd-timesyncd | time-daemon",
          "util-linux (>= 2.27.1)",
          "mount (>= 2.26)",
          "adduser"
        ],
        "pre_depends": [
          "libblkid1 (>= 2.24)",
          "libc6 (>= 2.28)",
          "libgcrypt20 (>= 1.8.0)",
          "liblz4-1 (>= 0.0~r122)",
          "liblzma5 (>= 5.1.1alpha+20120614)",
          "libselinux1 (>= 2.1.9)",
          "libzstd1 (>= 1.4.0)"
        ],
        "recommends": [
          "dbus"
        ],
        "suggests": [
          "systemd-container",
          "policykit-1"
        ],
        "conflicts": [
          "consolekit",
          "libpam-ck-connector"
        ],
        "breaks": [
          "python-dbusmock (<< 0.18)",
          "python3-dbusmock (<< 0.18)",
          "resolvconf (<< 1.83~)",
          "systemd-shim (<< 10-4~)",
          "udev (<< 247~)"
        ],
        "description_md5": "19399579cbc0c47a303288bf15eadcd4",
        "multi_arch": "foreign",
        "homepage": "https://www.freedesktop.org/wiki/Software/systemd",
        "section": "admin",
        "priority": "important",
        "filename": "pool/main/s/systemd/systemd_247.3-6~bpo10+1_amd64.deb",
        "size": 4382056,
        "sha256": "2035450655ad02faa0f75dc952128b503e51df5795c67273c0f6...",
        "description": "system and service manager  systemd is a system..."
      },
      ...
    ]

    $ apt-cache show | jc --apt-cache-show -p -r
    [
      {
        "package": "systemd",
        "version": "247.3-6~bpo10+1",
        "installed_size": "16121",
        "maintainer": "Debian systemd Maintainers <pkg-systemd-maintain...",
        "architecture": "amd64",
        "depends": "libacl1 (>= 2.2.23), libapparmor1 (>= 2.13), libaud...",
        "pre_depends": "libblkid1 (>= 2.24), libc6 (>= 2.28), libgcrypt...",
        "recommends": "dbus",
        "suggests": "systemd-container, policykit-1",
        "conflicts": "consolekit, libpam-ck-connector",
        "breaks": "python-dbusmock (<< 0.18), python3-dbusmock (<< 0.18...",
        "description_md5": "19399579cbc0c47a303288bf15eadcd4",
        "multi_arch": "foreign",
        "homepage": "https://www.freedesktop.org/wiki/Software/systemd",
        "section": "admin",
        "priority": "important",
        "filename": "pool/main/s/systemd/systemd_247.3-6~bpo10+1_amd64.deb",
        "size": "4382056",
        "sha256": "2035450655ad02faa0f75dc952128b503e51df5795c67273c0f6...",
        "description": "system and service manager  systemd is a system..."
      },
      ...
    ]
"""
from typing import List
from jc.jc_types import JSONDictType
import jc.parsers.rpm_qi as rpm_qi


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.0'
    description = '`apt-cache show` command parser'
    author = 'Kelly Brazil'
    author_email = 'kellyjonbrazil@gmail.com'
    details = 'Using the rpm-qi parser'
    compatible = ['linux']
    tags = ['command']
    magic_commands = ['apt-cache show']


__version__ = info.version


def parse(
    data: str,
    raw: bool = False,
    quiet: bool = False
) -> List[JSONDictType]:
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        List of Dictionaries. Raw or processed structured data.
    """
    # This parser is an alias of rpm_qi.py
    rpm_qi.info = info  # type: ignore
    rpm_qi.__name__ = __name__
    return rpm_qi.parse(data, raw, quiet)



r"""jc - JSON Convert HTTP headers parser

Converts HTTP request and response headers into a list of dictionaries.
Well-known headers are processed to allow multiple instances which are
aggregated into an array along with any comma-separated values. Integer,
float, and datetimes are converted where defined in the specifications.

https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers
https://datatracker.ietf.org/doc/html/rfc2616
https://datatracker.ietf.org/doc/html/rfc3229
https://datatracker.ietf.org/doc/html/rfc7089
https://datatracker.ietf.org/doc/html/rfc7231
https://datatracker.ietf.org/doc/html/rfc5789

If you are converting HTTP headers from `curl` verbose output, use the
`curl-head` parser which will strip the `>` and `<` characters and remove
non-header lines that begin with `*`.

Usage (cli):

    $ cat headers.txt | jc --http-headers

Usage (module):

    import jc
    result = jc.parse('http_headers', http_headers_output)

Schema:

    [
      {
        "_type":                                string,  # request or response
        "_request_method":                      string,
        "_request_uri":                         string,
        "_request_version":                     string,
        "_response_version":                    string,
        "_response_status":                     integer,
        "_response_reason":                     string or null,
        "<header>":                             string,

        # well-known headers:

        "accept": [
                                                string
        ],
        "accept-ch": [
                                                string
        ],
        "accept-ch-lifetime":                   integer,
        "accept-encoding": [
                                                string
        ],
        "accept-language": [
                                                string
        ],
        "accept-patch": [
                                                string
        ],
        "accept-post": [
                                                string
        ],
        "accept-ranges": [
                                                string
        ],
        "access-control-allow-headers": [
                                                string
        ],
        "access-control-allow-methods": [
                                                string
        ],
        "access-control-expose-headers": [
                                                string
        ],
        "access-control-max-age":               integer,
        "access-control-request-headers": [
                                                string
        ],
        "age":                                  integer,
        "allow": [
                                                string
        ],
        "alt-svc": [
                                                string
        ],
        "cache-control": [
                                                string
        ],
        "clear-site-data": [
                                                string
        ],
        "connection": [
                                                string
        ],
        "content-encoding": [
                                                string
        ],
        "content-dpr":                          integer,
        "content-language": [
                                                string
        ],
        "content-length":                       integer,
        "content-security-policy": [
                                                string
        ],
        "content-security-policy-report-only": [
                                                string
        ],
        "cookie": [
                                                string
        ],
        "critical-ch": [
                                                string
        ],
        "date":                                 string,
        "date_epoch_utc":                       integer,
        "expect-ct": [
                                                string
        ],
        "expires":                              string,
        "expires_epoch_utc":                    integer,
        "device-memory":                        integer,
        "downlink":                             integer,
        "dpr":                                  integer,
        "forwarded": [
                                                string
        ],
        "if-match": [
                                                string
        ],
        "if-modified-since":                    string,
        "if-modified-since_epoch_utc":          integer,
        "if-none-match": [
                                                string
        ],
        "if-range":                             string,
        "if-range_epoch_utc":                   integer,
        "if-unmodified-since":                  string,
        "if-unmodified-since_epoch_utc":        integer,
        "im": [
                                                string
        ],
        "keep-alive": [
                                                string
        ],
        "large-allocation":                     integer,
        "last-modified":                        string,
        "last-modified_epoch_utc":              integer,
        "link": [
                                                string
        ],
        "max-forwards":                         integer,
        "memento-datetime":                     string,
        "memento-datetime_epoch_utc":           integer,
        "permissions-policy": [
                                                string
        ],
        "permissions-policy-report-only": [
                                                string
        ],
        "pragma": [
                                                string
        ],
        "proxy-authenticate": [
                                                string
        ],
        "reporting-endpoints": [
                                                string
        ],
        "retry-after":                          string,
        "retry-after_epoch_utc":                integer,
        "rtt":                                  integer,
        "sec-ch-ua": [
                                                string
        ],
        "sec-ch-ua-full-version-list": [
                                                string
        ],
        "server": [
                                                string
        ],
        "server-timing": [
                                                string
        ],
        "set-cookie": [
                                                string
        ],
        "timing-allow-origin": [
                                                string
        ],
        "trailer": [
                                                string
        ],
        "transfer-encoding": [
                                                string
        ],
        "upgrade": [
                                                string
        ],
        "upgrade-insecure-requests":            integer,
        "vary": [
                                                string
        ],
        "via": [
                                                string
        ],
        "warning": [
                                                string
        ],
        "www-authenticate": [
                                                string
        ],
        "x-cache-hits": [
                                                integer
        ],
        "x-content-duration":                   float
      }
    ]

Examples:

    $ cat headers.txt | jc --http-headers -p
    [
      {
        "_type": "request",
        "_request_method": "HEAD",
        "_request_uri": "/",
        "_request_version": "HTTP/1.1",
        "host": "example.com",
        "user-agent": "curl/8.1.2",
        "accept": [
          "*/*"
        ]
      },
      {
        "_type": "response",
        "_response_version": "HTTP/1.1",
        "_response_status": 200,
        "_response_reason": [
          "OK"
        ],
        "accept-ranges": [
          "bytes"
        ],
        "age": 140203,
        "cache-control": [
          "max-age=604800"
        ],
        "content-type": "text/html; charset=UTF-8",
        "date": "Sun, 04 Feb 2024 02:25:07 GMT",
        "etag": "\"3147526947\"",
        "expires": "Sun, 11 Feb 2024 02:25:07 GMT",
        "last-modified": "Thu, 17 Oct 2019 07:18:26 GMT",
        "server": [
          "ECS (sac/252F)"
        ],
        "x-cache": "HIT",
        "content-length": 1256,
        "date_epoch_utc": 1707013507,
        "expires_epoch_utc": 1707618307,
        "last-modified_epoch_utc": 1571296706
      }
    ]

    $ cat headers.txt | jc --http-headers -p -r
    [
      {
        "_type": "request",
        "_request_method": "HEAD",
        "_request_uri": "/",
        "_request_version": "HTTP/1.1",
        "host": "example.com",
        "user-agent": "curl/8.1.2",
        "accept": [
          "*/*"
        ]
      },
      {
        "_type": "response",
        "_response_version": "HTTP/1.1",
        "_response_status": 200,
        "_response_reason": [
          "OK"
        ],
        "accept-ranges": [
          "bytes"
        ],
        "age": "140203",
        "cache-control": [
          "max-age=604800"
        ],
        "content-type": "text/html; charset=UTF-8",
        "date": "Sun, 04 Feb 2024 02:25:07 GMT",
        "etag": "\"3147526947\"",
        "expires": "Sun, 11 Feb 2024 02:25:07 GMT",
        "last-modified": "Thu, 17 Oct 2019 07:18:26 GMT",
        "server": [
          "ECS (sac/252F)"
        ],
        "x-cache": "HIT",
        "content-length": "1256"
      }
    ]
"""
from typing import List, Dict
from jc.jc_types import JSONDictType
import jc.utils


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.0'
    description = 'HTTP headers parser'
    author = 'Kelly Brazil'
    author_email = 'kellyjonbrazil@gmail.com'
    compatible = ['linux', 'darwin', 'cygwin', 'win32', 'aix', 'freebsd']
    tags = ['standard', 'file']


__version__ = info.version

METHODS = {'connect', 'delete', 'get', 'head', 'options', 'patch', 'post', 'put', 'trace'}

INT_HEADERS = {
    'accept-ch-lifetime',
    'access-control-max-age',
    'age',
    'content-dpr',
    'content-length',
    'device-memory',
    'downlink',
    'dpr',
    'large-allocation',
    'max-forwards',
    'rtt',
    'upgrade-insecure-requests'
}

FLOAT_HEADERS = {
    'x-content-duration'
}

DT_HEADERS = {
    'date',
    'if-modified-since',
    'if-unmodified-since',
    'last-modified',
    'memento-datetime'
}

DT_OR_INT_HEADERS = {
    'expires',
    'retry-after'
}

DT_OR_STR_HEADERS = {
    'if-range'
}

MULTI_HEADERS = {
    'content-security-policy',
    'content-security-policy-report-only',
    'cookie',
    'set-cookie'
}

SPLIT_AND_MULTI_HEADERS = {
    'accept',
    'accept-ch',
    'accept-encoding',
    'accept-language',
    'accept-patch',
    'accept-post',
    'accept-ranges',
    'access-control-allow-headers',
    'access-control-allow-methods',
    'access-control-expose-headers',
    'access-control-request-headers',
    'allow',
    'alt-svc',
    'cache-control',
    'clear-site-data',
    'connection',
    'content-encoding',
    'content-language',
    'critical-ch',
    'expect-ct',
    'forwarded',
    'if-match',
    'if-none-match',
    'im',
    'keep-alive',
    'link',
    'permissions-policy',
    'permissions-policy-report-only',
    'pragma',
    'proxy-authenticate',
    'reporting-endpoints',
    'sec-ch-ua',
    'sec-ch-ua-full-version-list',
    'server',
    'server-timing',
    'timing-allow-origin',
    'trailer',
    'transfer-encoding',
    'upgrade',
    'vary',
    'via',
    'warning',
    'www-authenticate',
    'x-cache-hits'
}

def _process(proc_data: List[JSONDictType]) -> List[JSONDictType]:
    """
    Final processing to conform to the schema.

    Parameters:

        proc_data:   (List of Dictionaries) raw structured data to process

    Returns:

        List of Dictionaries. Structured to conform to the schema.
    """
    for item in proc_data:

        for key in item.copy():
            if key in INT_HEADERS:
                item[key] = jc.utils.convert_to_int(item[key])

            if key in FLOAT_HEADERS:
                item[key] = jc.utils.convert_to_float(item[key])

            if key in DT_HEADERS or key in DT_OR_STR_HEADERS:
                timestamp = jc.utils.timestamp(item[key], format_hint=(3500,)).utc
                if timestamp:
                    item[key + '_epoch_utc'] = timestamp

            if key in DT_OR_INT_HEADERS:
                timestamp = jc.utils.timestamp(item[key], format_hint=(3500,)).utc
                if timestamp:
                    item[key + '_epoch_utc'] = timestamp
                if item[key].isnumeric():
                    item[key] = jc.utils.convert_to_int(item[key])

        # special handling
        if 'x-cache-hits' in item:
            item['x-cache-hits'] = [jc.utils.convert_to_int(val) for val in item['x-cache-hits']]

    return proc_data


def parse(
    data: str,
    raw: bool = False,
    quiet: bool = False
) -> List[JSONDictType]:
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        List of Dictionaries. Raw or processed structured data.
    """
    jc.utils.compatibility(__name__, info.compatible, quiet)
    jc.utils.input_type_check(data)

    raw_output: List[Dict] = []
    output_object: Dict = {}

    if jc.utils.has_data(data):

        for line in filter(None, data.splitlines()):

            first_word = line.split(maxsplit=1)[0]
            first_word = first_word.rstrip(':')
            first_word = first_word.lower()

            if first_word in METHODS:
                if output_object:
                    raw_output.append(output_object)

                method, uri, version = line.split(maxsplit=2)
                output_object = {}
                output_object['_type'] = 'request'
                output_object['_request_method'] = method
                output_object['_request_uri'] = uri
                output_object['_request_version'] = version
                continue

            if first_word.startswith('http/'):
                if output_object:
                    raw_output.append(output_object)

                reason = None
                version, status, *reason = line.split(maxsplit=2)
                output_object = {}
                output_object['_type'] = 'response'
                output_object['_response_version'] = version
                output_object['_response_status'] = int(status)
                output_object['_response_reason'] = reason or None
                continue

            if first_word in SPLIT_AND_MULTI_HEADERS:
                key, value = line.split(': ', maxsplit=1)
                key = key.lower()
                value_list = value.split(',')
                value_list = [x.strip() for x in value_list]
                if key in output_object:
                    output_object[key].extend(value_list)
                else:
                    output_object[key] = []
                    output_object[key].extend(value_list)
                continue

            if first_word in MULTI_HEADERS:
                key, value = line.split(': ', maxsplit=1)
                key = key.lower()
                if key in output_object:
                    output_object[key].append(value)
                else:
                    output_object[key] = []
                    output_object[key].append(value)
                continue

            # All other headers
            key, value = line.split(': ', maxsplit=1)
            key = key.lower()
            output_object[key] = value

    if output_object:
        raw_output.append(output_object)

    return raw_output if raw else _process(raw_output)



from copy import deepcopy
import unittest
from typing import Generator
from types import ModuleType
import jc.lib
import jc.parsers.csv as csv_parser


class MyTests(unittest.TestCase):
    def test_lib_get_parser_string(self):
        p = jc.lib.get_parser('arp')
        self.assertIsInstance(p, ModuleType)

    def test_lib_get_parser_broken_parser(self):
        """get_parser substitutes the disabled_parser if a parser is broken"""
        broken = jc.lib.get_parser('broken_parser')
        disabled = jc.lib.get_parser('disabled_parser')
        self.assertIs(broken, disabled)

    def test_lib_get_parser_module(self):
        p = jc.lib.get_parser(csv_parser)
        self.assertIsInstance(p, ModuleType)

    def test_lib_parse_csv(self):
        data = {
            '': [],
            'a,b,c\n1,2,3': [{'a':'1', 'b':'2', 'c':'3'}]
        }

        for test_data, expected_output in data.items():
            self.assertEqual(jc.lib.parse('csv', test_data), expected_output)

    def test_lib_parse_csv_s_is_generator(self):
        self.assertIsInstance(jc.lib.parse('csv_s', 'a,b,c\n1,2,3'), Generator)

    def test_lib_parse_kv(self):
        data = {
            '': {},
            'a=1\nb=2\nc=3': {'a':'1', 'b':'2', 'c':'3'}
        }

        for test_data, expected_output in data.items():
            self.assertEqual(jc.lib.parse('kv', test_data), expected_output)

    def test_lib_parser_mod_list_is_list(self):
        self.assertIsInstance(jc.lib.parser_mod_list(), list)

    def test_lib_parser_mod_list_contains_csv(self):
        self.assertTrue('csv' in jc.lib.parser_mod_list())

    def test_lib_parser_mod_list_length(self):
        self.assertGreaterEqual(len(jc.lib.parser_mod_list()), 80)

    def test_lib_parser_info_is_dict(self):
        self.assertIsInstance(jc.lib.parser_info('csv'), dict)

    def test_lib_parser_info_csv(self):
        self.assertTrue(jc.lib.parser_info('csv')['name'] == 'csv')

    def test_lib_all_parser_info_is_list_of_dicts(self):
        self.assertIsInstance(jc.lib.all_parser_info(), list)
        self.assertIsInstance(jc.lib.all_parser_info()[0], dict)

    def test_lib_all_parser_info_contains_csv(self):
        p_list = []
        for p in jc.lib.all_parser_info():
            p_list.append(p['name'])
        self.assertTrue('csv' in p_list)

    def test_lib_all_parser_info_length(self):
        self.assertGreaterEqual(len(jc.lib.all_parser_info()), 80)

    def test_lib_all_parser_hidden_length(self):
        reg_length = len(jc.lib.all_parser_info())
        hidden_length = len(jc.lib.all_parser_info(show_hidden=True))
        self.assertGreater(hidden_length, reg_length)

    def test_lib_plugin_parser_mod_list_is_list(self):
        self.assertIsInstance(jc.lib.plugin_parser_mod_list(), list)

    def test_lib_plugin_parser_mod_list_length_is_zero(self):
        """Ensure there are no plugin parsers present during test/build."""
        self.assertEqual(len(jc.lib.plugin_parser_mod_list()), 0)

    def test_lib_cliname_to_modname(self):
        self.assertEqual(jc.lib._cliname_to_modname('module-name'), 'module_name')

    def test_lib_argumentname_to_modname(self):
        self.assertEqual(jc.lib._cliname_to_modname('--module-name'), 'module_name')

    def test_lib_modname_to_cliname(self):
        self.assertEqual(jc.lib._modname_to_cliname('module_name'), 'module-name')

    def test_lib_all_parser_info_show_deprecated(self):
        # save old state
        old_parsers = deepcopy(jc.lib.parsers)
        old_get_parser = deepcopy(jc.lib.get_parser)

        # mock data
        class mock_parser_info:
            version = "1.1"
            description = "`deprecated` command parser"
            author = "nobody"
            author_email = "nobody@gmail.com"
            compatible = ["linux", "darwin"]
            magic_commands = ["deprecated"]
            deprecated = True

        class mock_parser:
            info = mock_parser_info
            def parse():
                pass

        jc.lib.parsers = ['deprecated']
        jc.lib.get_parser = lambda x: mock_parser  # type: ignore
        result = jc.lib.all_parser_info(show_deprecated=True)

        # reset
        jc.lib.parsers = old_parsers
        jc.lib.get_parser = old_get_parser

        self.assertEqual(len(result), 1)

    def test_lib_all_parser_info_show_hidden(self):
        # save old state
        old_parsers = deepcopy(jc.lib.parsers)
        old_get_parser = deepcopy(jc.lib.get_parser)

        # mock data
        class mock_parser_info:
            version = "1.1"
            description = "`deprecated` command parser"
            author = "nobody"
            author_email = "nobody@gmail.com"
            compatible = ["linux", "darwin"]
            magic_commands = ["deprecated"]
            hidden = True

        class mock_parser:
            info = mock_parser_info
            def parse():
                pass

        jc.lib.parsers = ['deprecated']
        jc.lib.get_parser = lambda x: mock_parser  # type: ignore
        result = jc.lib.all_parser_info(show_hidden=True)

        # reset
        jc.lib.parsers = old_parsers
        jc.lib.get_parser = old_get_parser

        self.assertEqual(len(result), 1)


if __name__ == '__main__':
    unittest.main()


r"""jc - JSON Convert `gpg --with-colons` command output parser

Usage (cli):

    $ gpg --with-colons --show-keys file.gpg | jc --gpg

or

    $ jc gpg --with-colons --show-keys file.gpg

Usage (module):

    import jc
    result = jc.parse('gpg', gpg_command_output)

Schema:

Field definitions from https://git.gnupg.org/cgi-bin/gitweb.cgi?p=gnupg.git;a=blob_plain;f=doc/DETAILS

> Note: Number values are not converted to integers because many field
> specifications are overloaded and future augmentations are implied in the
> documentation.

    [
      {
        "type":                             string,
        "validity":                         string,
        "key_length":                       string,
        "pub_key_alg":                      string,
        "key_id":                           string,
        "creation_date":                    string,
        "expiration_date":                  string,
        "certsn_uidhash_trustinfo":         string,
        "owner_trust":                      string,
        "user_id":                          string,
        "signature_class":                  string,
        "key_capabilities":                 string,
        "cert_fingerprint_other":           string,
        "flag":                             string,
        "token_sn":                         string,
        "hash_alg":                         string,
        "curve_name":                       string,
        "compliance_flags":                 string,
        "last_update_date":                 string,
        "origin":                           string,
        "comment":                          string,
        "index":                            string,  # [0]
        "bits":                             string,  # [0]
        "value":                            string,  # [0]
        "version":                          string,  # [1], [4]
        "signature_count":                  string,  # [1]
        "encryption_count":                 string,  # [1]
        "policy":                           string,  # [1]
        "signature_first_seen":             string,  # [1]
        "signature_most_recent_seen":       string,  # [1]
        "encryption_first_done":            string,  # [1]
        "encryption_most_recent_done":      string,  # [1]
        "staleness_reason":                 string,  # [2]
        "trust_model":                      string,  # [2]
        "trust_db_created":                 string,  # [2]
        "trust_db_expires":                 string,  # [2]
        "marginally_trusted_users":         string,  # [2]
        "completely_trusted_users":         string,  # [2]
        "cert_chain_max_depth":             string,  # [2]
        "subpacket_number":                 string,  # [3]
        "hex_flags":                        string,  # [3]
        "subpacket_length":                 string,  # [3]
        "subpacket_data":                   string,  # [3]
        "pubkey":                           string,  # [4]
        "cipher":                           string,  # [4]
        "digest":                           string,  # [4]
        "compress":                         string,  # [4]
        "group":                            string,  # [4]
        "members":                          string,  # [4]
        "curve_names":                      string,  # [4]
      }
    ]

    All blank values are converted to null/None.

    [0] for 'pkd' type
    [1] for 'tfs' type
    [2] for 'tru' type
    [3] for 'skp' type
    [4] for 'cfg' type

Examples:

    $ gpg --with-colons --show-keys file.gpg | jc --gpg -p
    [
      {
        "type": "pub",
        "validity": "f",
        "key_length": "1024",
        "pub_key_alg": "17",
        "key_id": "6C7EE1B8621CC013",
        "creation_date": "899817715",
        "expiration_date": "1055898235",
        "certsn_uidhash_trustinfo": null,
        "owner_trust": "m",
        "user_id": null,
        "signature_class": null,
        "key_capabilities": "scESC",
        "cert_fingerprint_other": null,
        "flag": null,
        "token_sn": null,
        "hash_alg": null,
        "curve_name": null,
        "compliance_flags": null,
        "last_update_date": null,
        "origin": null,
        "comment": null
      },
      ...
    ]
"""
from typing import List, Dict, Optional
import jc.utils


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.0'
    description = '`gpg --with-colons` command parser'
    author = 'Kelly Brazil'
    author_email = 'kellyjonbrazil@gmail.com'
    compatible = ['linux']
    magic_commands = ['gpg --with-colons']
    tags = ['command']


__version__ = info.version


def _process(proc_data: List[Dict]) -> List[Dict]:
    """
    Final processing to conform to the schema.

    Parameters:

        proc_data:   (List of Dictionaries) raw structured data to process

    Returns:

        List of Dictionaries. Structured to conform to the schema.
    """
    return proc_data


def _list_get(my_list: List, index: int, default_val=None) -> Optional[str]:
    """get a list value or return None/default value if out of range."""
    if index <= len(my_list) - 1:
        return my_list[index] or None

    return default_val


def parse(
    data: str,
    raw: bool = False,
    quiet: bool = False
) -> List[Dict]:
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        List of Dictionaries. Raw or processed structured data.
    """
    jc.utils.compatibility(__name__, info.compatible, quiet)
    jc.utils.input_type_check(data)

    raw_output: List = []

    if jc.utils.has_data(data):

        for line in filter(None, data.splitlines()):
            values = line.split(':')
            temp_obj = {
                'type': _list_get(values, 0),
                'validity': _list_get(values, 1),
                'key_length': _list_get(values, 2),
                'pub_key_alg': _list_get(values, 3),
                'key_id': _list_get(values, 4),
                'creation_date': _list_get(values, 5),
                'expiration_date': _list_get(values, 6),
                'certsn_uidhash_trustinfo': _list_get(values, 7),
                'owner_trust': _list_get(values, 8),
                'user_id': _list_get(values, 9),
                'signature_class': _list_get(values, 10),
                'key_capabilities': _list_get(values, 11),
                'cert_fingerprint_other': _list_get(values, 12),
                'flag': _list_get(values, 13),
                'token_sn': _list_get(values, 14),
                'hash_alg': _list_get(values, 15),
                'curve_name': _list_get(values, 16),
                'compliance_flags': _list_get(values, 17),
                'last_update_date': _list_get(values, 18),
                'origin': _list_get(values, 19),
                'comment': _list_get(values, 20)
            }

            # field mappings change for special types: pkd, tfs, tru, skp, cfg

            if temp_obj['type'] == 'pkd':
                # pkd:0:1024:B665B1435F4C2 .... FF26ABB:
                #     !  !   !-- the value
                #     !  !------ for information number of bits in the value
                #     !--------- index (eg. DSA goes from 0 to 3: p,q,g,y)
                line_obj = {
                    'type': temp_obj['type'],
                    'index': temp_obj['validity'],
                    'bits': temp_obj['key_length'],
                    'value': temp_obj['pub_key_alg']
                }

            elif temp_obj['type'] == 'tfs':
                # - Field 2 :: tfs record version (must be 1)
                # - Field 3 :: validity -  A number with validity code.
                # - Field 4 :: signcount - The number of signatures seen.
                # - Field 5 :: encrcount - The number of encryptions done.
                # - Field 6 :: policy - A string with the policy
                # - Field 7 :: signture-first-seen - a timestamp or 0 if not known.
                # - Field 8 :: signature-most-recent-seen - a timestamp or 0 if not known.
                # - Field 9 :: encryption-first-done - a timestamp or 0 if not known.
                # - Field 10 :: encryption-most-recent-done - a timestamp or 0 if not known.
                line_obj = {
                    'type': temp_obj['type'],
                    'version': temp_obj['validity'],
                    'validity': temp_obj['key_length'],
                    'signature_count': temp_obj['pub_key_alg'],
                    'encryption_count': temp_obj['key_id'],
                    'policy': temp_obj['creation_date'],
                    'signature_first_seen': temp_obj['expiration_date'],
                    'signature_most_recent_seen': temp_obj['certsn_uidhash_trustinfo'],
                    'encryption_first_done': temp_obj['owner_trust'],
                    'encryption_most_recent_done': temp_obj['user_id']
                }

            elif temp_obj['type'] == 'tru':
                # tru:o:0:1166697654:1:3:1:5
                # - Field 2 :: Reason for staleness of trust.
                # - Field 3 :: Trust model
                # - Field 4 :: Date trustdb was created in seconds since Epoch.
                # - Field 5 :: Date trustdb will expire in seconds since Epoch.
                # - Field 6 :: Number of marginally trusted users to introduce a new key signer.
                # - Field 7 :: Number of completely trusted users to introduce a new key signer.
                # - Field 8 :: Maximum depth of a certification chain.
                line_obj = {
                    'type': temp_obj['type'],
                    'staleness_reason': temp_obj['validity'],
                    'trust_model': temp_obj['key_length'],
                    'trust_db_created': temp_obj['pub_key_alg'],
                    'trust_db_expires': temp_obj['key_id'],
                    'marginally_trusted_users': temp_obj['creation_date'],
                    'completely_trusted_users': temp_obj['expiration_date'],
                    'cert_chain_max_depth': temp_obj['certsn_uidhash_trustinfo']
                }

            elif temp_obj['type'] == 'skp':
                # - Field 2 :: Subpacket number as per RFC-4880 and later.
                # - Field 3 :: Flags in hex.
                # - Field 4 :: Length of the subpacket.
                # - Field 5 :: The subpacket data.
                line_obj = {
                    'type': temp_obj['type'],
                    'subpacket_number': temp_obj['validity'],
                    'hex_flags': temp_obj['key_length'],
                    'subpacket_length': temp_obj['pub_key_alg'],
                    'subpacket_data': temp_obj['key_id']
                }

            elif temp_obj['type'] == 'cfg':

                # there are several 'cfg' formats

                if temp_obj['validity'] == 'version':
                    # cfg:version:1.3.5
                    line_obj = {
                        'type': temp_obj['type'],
                        'version': temp_obj['key_length']
                    }

                elif temp_obj['validity'] == 'pubkey':
                    # cfg:pubkey:1;2;3;16;17
                    line_obj = {
                        'type': temp_obj['type'],
                        'pubkey': temp_obj['key_length']
                    }

                elif temp_obj['validity'] == 'cipher':
                    # cfg:cipher:2;3;4;7;8;9;10
                    line_obj = {
                        'type': temp_obj['type'],
                        'cipher': temp_obj['key_length']
                    }

                elif temp_obj['validity'] == 'digest':
                    # cfg:digest:1;2;3;8;9;10
                    line_obj = {
                        'type': temp_obj['type'],
                        'digest': temp_obj['key_length']
                    }

                elif temp_obj['validity'] == 'compress':
                    # cfg:compress:0;1;2;3
                    line_obj = {
                        'type': temp_obj['type'],
                        'compress': temp_obj['key_length']
                    }

                elif temp_obj['validity'] == 'group':
                    # cfg:group:mynames:patti;joe;0x12345678;paige
                    line_obj = {
                        'type': temp_obj['type'],
                        'group': temp_obj['key_length'],
                        'members': temp_obj['pub_key_alg']
                    }

                elif temp_obj['validity'] == 'curve':
                    # cfg:curve:ed25519;nistp256;nistp384;nistp521
                    line_obj = {
                        'type': temp_obj['type'],
                        'curve_names': temp_obj['key_length']
                    }

            else:
                line_obj = temp_obj

            raw_output.append(line_obj)

    return raw_output if raw else _process(raw_output)



import os
import unittest
import json
import jc.parsers.nmcli
from jc.exceptions import ParseError

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):

    # input
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-connection-all.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_connection_all = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-connection-show-ens33.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_connection_show_ens33 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-connection.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_connection = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-all.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_all = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-show-ens33.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_show_ens33 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-show-lo.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_show_lo = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-show.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_show = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-general-all.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_general_all = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-general-permissions.out'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_general_permissions = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/fedora32/nmcli-connection-show-ens33.out'), 'r', encoding='utf-8') as f:
        fedora32_nmcli_connection_show_ens33 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/fedora32/nmcli-device-show-ens33.out'), 'r', encoding='utf-8') as f:
        fedora32_nmcli_device_show_ens33 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/fedora32/nmcli-device-show.out'), 'r', encoding='utf-8') as f:
        fedora32_nmcli_device_show = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-config.out'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_config = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-config-blank.out'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_config_blank = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-port-config.out'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_port_config = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-port-config-blank.out'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_port_config_blank = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-and-team-port-config.out'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_and_team_port_config = f.read()


    # output
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-connection-all.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_connection_all_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-connection-show-ens33.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_connection_show_ens33_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-connection.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_connection_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-all.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_all_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-show-ens33.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_show_ens33_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-show-lo.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_show_lo_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device-show.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_show_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-device.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_device_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-general-all.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_general_all_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/nmcli-general-permissions.json'), 'r', encoding='utf-8') as f:
        centos_7_7_nmcli_general_permissions_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/fedora32/nmcli-connection-show-ens33.json'), 'r', encoding='utf-8') as f:
        fedora32_nmcli_connection_show_ens33_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/fedora32/nmcli-device-show-ens33.json'), 'r', encoding='utf-8') as f:
        fedora32_nmcli_device_show_ens33_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/fedora32/nmcli-device-show.json'), 'r', encoding='utf-8') as f:
        fedora32_nmcli_device_show_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-config.json'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_config_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-config-blank.json'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_config_blank_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-port-config.json'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_port_config_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-port-config-blank.json'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_port_config_blank_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/nmcli-team-and-team-port-config.json'), 'r', encoding='utf-8') as f:
        generic_nmcli_team_and_team_port_config_json = json.loads(f.read())



    def test_nmcli_nodata(self):
        """
        Test 'nmcli' with no data
        """
        self.assertEqual(jc.parsers.nmcli.parse('', quiet=True), [])

    def test_nmcli_centos_7_7(self):
        """
        Test 'nmcli' on Centos 7.7 - this should raise a ParseError exception
        """
        self.assertRaises(ParseError, jc.parsers.nmcli.parse, self.centos_7_7_nmcli, quiet=True)

    def test_nmcli_connection_all_centos_7_7(self):
        """
        Test 'nmcli -f all connection' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_connection_all, quiet=True), self.centos_7_7_nmcli_connection_all_json)

    def test_nmcli_connection_show_ens33_centos_7_7(self):
        """
        Test 'nmcli connection show ens33' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_connection_show_ens33, quiet=True), self.centos_7_7_nmcli_connection_show_ens33_json)

    def test_nmcli_connection_centos_7_7(self):
        """
        Test 'nmcli connection' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_connection, quiet=True), self.centos_7_7_nmcli_connection_json)

    def test_nmcli_device_all_centos_7_7(self):
        """
        Test 'nmcli -f all device' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_device_all, quiet=True), self.centos_7_7_nmcli_device_all_json)

    def test_nmcli_device_show_ens33_centos_7_7(self):
        """
        Test 'nmcli device show ens33' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_device_show_ens33, quiet=True), self.centos_7_7_nmcli_device_show_ens33_json)

    def test_nmcli_device_show_lo_centos_7_7(self):
        """
        Test 'nmcli device show lo' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_device_show_lo, quiet=True), self.centos_7_7_nmcli_device_show_lo_json)

    def test_nmcli_device_show_centos_7_7(self):
        """
        Test 'nmcli device show' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_device_show, quiet=True), self.centos_7_7_nmcli_device_show_json)

    def test_nmcli_device_centos_7_7(self):
        """
        Test 'nmcli device' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_device, quiet=True), self.centos_7_7_nmcli_device_json)

    def test_nmcli_general_all_centos_7_7(self):
        """
        Test 'nmcli -f all general' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_general_all, quiet=True), self.centos_7_7_nmcli_general_all_json)

    def test_nmcli_general_permissions_centos_7_7(self):
        """
        Test 'nmcli general permissions' on Centos 7.7
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.centos_7_7_nmcli_general_permissions, quiet=True), self.centos_7_7_nmcli_general_permissions_json)

    def test_nmcli_connection_show_ens33_fedora32(self):
        """
        Test 'nmcli connection show ens33' on fedora32
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.fedora32_nmcli_connection_show_ens33, quiet=True), self.fedora32_nmcli_connection_show_ens33_json)

    def test_nmcli_device_show_ens33_fedora32(self):
        """
        Test 'nmcli device show ens33' on fedora32
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.fedora32_nmcli_device_show_ens33, quiet=True), self.fedora32_nmcli_device_show_ens33_json)

    def test_nmcli_device_show_fedora32(self):
        """
        Test 'nmcli device show' on fedora32
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.fedora32_nmcli_device_show, quiet=True), self.fedora32_nmcli_device_show_json)

    def test_nmcli_team_config(self):
        """
        Test nmcli with team.config JSON value
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.generic_nmcli_team_config, quiet=True), self.generic_nmcli_team_config_json)

    def test_nmcli_team_config_blank(self):
        """
        Test nmcli with blank team.config JSON value
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.generic_nmcli_team_config_blank, quiet=True), self.generic_nmcli_team_config_blank_json)

    def test_nmcli_team_port_config(self):
        """
        Test nmcli with team-port.config JSON value
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.generic_nmcli_team_port_config, quiet=True), self.generic_nmcli_team_port_config_json)

    def test_nmcli_team_port_config_blank(self):
        """
        Test nmcli with blank team-port.config JSON value
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.generic_nmcli_team_port_config_blank, quiet=True), self.generic_nmcli_team_port_config_blank_json)

    def test_nmcli_team_and_team_port_config_blank(self):
        """
        Test nmcli with both team.config and team-port.config JSON value
        """
        self.assertEqual(jc.parsers.nmcli.parse(self.generic_nmcli_team_and_team_port_config, quiet=True), self.generic_nmcli_team_and_team_port_config_json)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
import json
import jc.parsers.yaml

THIS_DIR = os.path.dirname(os.path.abspath(__file__))

try:
    import ruamel.yaml
    RUAMELYAML_INSTALLED = True
except:
    RUAMELYAML_INSTALLED = False


@unittest.skipIf(not RUAMELYAML_INSTALLED, 'ruamel.yaml library not installed')
class MyTests(unittest.TestCase):

    # input
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/yaml-istio-sc.yaml'), 'r', encoding='utf-8') as f:
        generic_yaml_istio_sc = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/yaml-istio-sidecar.yaml'), 'r', encoding='utf-8') as f:
        generic_yaml_istio_sidecar = f.read()

    # output
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/yaml-istio-sc.json'), 'r', encoding='utf-8') as f:
        generic_yaml_istio_sc_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/generic/yaml-istio-sidecar.json'), 'r', encoding='utf-8') as f:
        generic_yaml_istio_sidecar_json = json.loads(f.read())


    def test_yaml_nodata(self):
        """
        Test the YAML parser with no data
        """
        self.assertEqual(jc.parsers.yaml.parse('', quiet=True), [])

    def test_yaml_istio_sc(self):
        """
        Test the Istio SC yaml file
        """
        self.assertEqual(jc.parsers.yaml.parse(self.generic_yaml_istio_sc, quiet=True), self.generic_yaml_istio_sc_json)

    def test_yaml_istio_sidecar(self):
        """
        Test the Istio Sidecar yaml file
        """
        self.assertEqual(jc.parsers.yaml.parse(self.generic_yaml_istio_sidecar, quiet=True), self.generic_yaml_istio_sidecar_json)

    def test_yaml_datetime(self):
        """
        Test yaml file with datetime object (should convert to a string)
        """
        data = 'deploymentTime: 2022-04-18T11:12:47'
        expected = [{"deploymentTime":"2022-04-18T11:12:47"}]
        self.assertEqual(jc.parsers.yaml.parse(data, quiet=True), expected)

    def test_yaml_equalsign(self):
        """
        Test yaml file with a value that starts with a literal equal sign "=" (should convert to a string)
        """
        data = 'key: ='
        expected = [{"key":"="}]
        self.assertEqual(jc.parsers.yaml.parse(data, quiet=True), expected)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
import json
from typing import Dict
from jc.parsers.pgpass import parse

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):
    f_in: Dict = {}
    f_json: Dict = {}

    @classmethod
    def setUpClass(cls):
        fixtures = {
            'pgpass': (
                'fixtures/generic/pgpass.txt',
                'fixtures/generic/pgpass.json')
        }

        for file, filepaths in fixtures.items():
            with open(os.path.join(THIS_DIR, filepaths[0]), 'r', encoding='utf-8') as a, \
                 open(os.path.join(THIS_DIR, filepaths[1]), 'r', encoding='utf-8') as b:
                cls.f_in[file] = a.read()
                cls.f_json[file] = json.loads(b.read())


    def test_pgpass_nodata(self):
        """
        Test 'pgpass' with no data
        """
        self.assertEqual(parse('', quiet=True), [])


    def test_pgpass(self):
        """
        Test postgreSQL password file
        """
        self.assertEqual(
            parse(self.f_in['pgpass'], quiet=True),
            self.f_json['pgpass']
        )


if __name__ == '__main__':
    unittest.main()



r"""jc - JSON Convert `/proc/<pid>/stat` file parser

Usage (cli):

    $ cat /proc/1/stat | jc --proc

or

    $ jc /proc/1/stat

or

    $ cat /proc/1/stat | jc --proc-pid-stat

Usage (module):

    import jc
    result = jc.parse('proc', proc_pid_stat_file)

or

    import jc
    result = jc.parse('proc_pid_stat', proc_pid_stat_file)

Schema:

    {
      "pid":                            integer,
      "comm":                           string,
      "state":                          string,
      "state_pretty":                   string,
      "ppid":                           integer,
      "pgrp":                           integer,
      "session":                        integer,
      "tty_nr":                         integer,
      "tpg_id":                         integer,
      "flags":                          integer,
      "minflt":                         integer,
      "cminflt":                        integer,
      "majflt":                         integer,
      "cmajflt":                        integer,
      "utime":                          integer,
      "stime":                          integer,
      "cutime":                         integer,
      "cstime":                         integer,
      "priority":                       integer,
      "nice":                           integer,
      "num_threads":                    integer,
      "itrealvalue":                    integer,
      "starttime":                      integer,
      "vsize":                          integer,
      "rss":                            integer,
      "rsslim":                         integer,
      "startcode":                      integer,
      "endcode":                        integer,
      "startstack":                     integer,
      "kstkeep":                        integer,
      "kstkeip":                        integer,
      "signal":                         integer,
      "blocked":                        integer,
      "sigignore":                      integer,
      "sigcatch":                       integer,
      "wchan":                          integer,
      "nswap":                          integer,
      "cnswap":                         integer,
      "exit_signal":                    integer,
      "processor":                      integer,
      "rt_priority":                    integer,
      "policy":                         integer,
      "delayacct_blkio_ticks":          integer,
      "guest_time":                     integer,
      "cguest_time":                    integer,
      "start_data":                     integer,
      "end_data":                       integer,
      "start_brk":                      integer,
      "arg_start":                      integer,
      "arg_end":                        integer,
      "env_start":                      integer,
      "env_end":                        integer,
      "exit_code":                      integer,
    }

Examples:

    $ cat /proc/1/stat | jc --proc -p
    {
      "pid": 1,
      "comm": "systemd",
      "state": "S",
      "ppid": 0,
      "pgrp": 1,
      "session": 1,
      "tty_nr": 0,
      "tpg_id": -1,
      "flags": 4194560,
      "minflt": 23478,
      "cminflt": 350218,
      "majflt": 99,
      "cmajflt": 472,
      "utime": 107,
      "stime": 461,
      "cutime": 2672,
      "cstime": 4402,
      "priority": 20,
      "nice": 0,
      "num_threads": 1,
      "itrealvalue": 0,
      "starttime": 128,
      "vsize": 174063616,
      "rss": 3313,
      "rsslim": 18446744073709551615,
      "startcode": 94188219072512,
      "endcode": 94188219899461,
      "startstack": 140725059845296,
      "kstkeep": 0,
      "kstkeip": 0,
      "signal": 0,
      "blocked": 671173123,
      "sigignore": 4096,
      "sigcatch": 1260,
      "wchan": 1,
      "nswap": 0,
      "cnswap": 0,
      "exit_signal": 17,
      "processor": 0,
      "rt_priority": 0,
      "policy": 0,
      "delayacct_blkio_ticks": 18,
      "guest_time": 0,
      "cguest_time": 0,
      "start_data": 94188220274448,
      "end_data": 94188220555504,
      "start_brk": 94188243599360,
      "arg_start": 140725059845923,
      "arg_end": 140725059845934,
      "env_start": 140725059845934,
      "env_end": 140725059846125,
      "exit_code": 0,
      "state_pretty": "Sleeping in an interruptible wait"
    }

    $ cat /proc/1/stat | jc --proc-pid-stat -p -r
    {
      "pid": 1,
      "comm": "systemd",
      "state": "S",
      "ppid": 0,
      "pgrp": 1,
      "session": 1,
      "tty_nr": 0,
      "tpg_id": -1,
      "flags": 4194560,
      "minflt": 23478,
      "cminflt": 350218,
      "majflt": 99,
      "cmajflt": 472,
      "utime": 107,
      "stime": 461,
      "cutime": 2672,
      "cstime": 4402,
      "priority": 20,
      "nice": 0,
      "num_threads": 1,
      "itrealvalue": 0,
      "starttime": 128,
      "vsize": 174063616,
      "rss": 3313,
      "rsslim": 18446744073709551615,
      "startcode": 94188219072512,
      "endcode": 94188219899461,
      "startstack": 140725059845296,
      "kstkeep": 0,
      "kstkeip": 0,
      "signal": 0,
      "blocked": 671173123,
      "sigignore": 4096,
      "sigcatch": 1260,
      "wchan": 1,
      "nswap": 0,
      "cnswap": 0,
      "exit_signal": 17,
      "processor": 0,
      "rt_priority": 0,
      "policy": 0,
      "delayacct_blkio_ticks": 18,
      "guest_time": 0,
      "cguest_time": 0,
      "start_data": 94188220274448,
      "end_data": 94188220555504,
      "start_brk": 94188243599360,
      "arg_start": 140725059845923,
      "arg_end": 140725059845934,
      "env_start": 140725059845934,
      "env_end": 140725059846125,
      "exit_code": 0
    }
"""
import re
from typing import Dict
import jc.utils


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.2'
    description = '`/proc/<pid>/stat` file parser'
    author = 'Kelly Brazil'
    author_email = 'kellyjonbrazil@gmail.com'
    compatible = ['linux']
    tags = ['file']
    hidden = True


__version__ = info.version


def _process(proc_data: Dict) -> Dict:
    """
    Final processing to conform to the schema.

    Parameters:

        proc_data:   (Dictionary) raw structured data to process

    Returns:

        Dictionary. Structured to conform to the schema.
    """
    state_map = {
        'R': 'Running',
        'S': 'Sleeping in an interruptible wait',
        'D': 'Waiting in uninterruptible disk sleep',
        'Z': 'Zombie',
        'T': 'Stopped (on a signal) or trace stopped',
        't': 'Tracing stop',
        'W': 'Paging',
        'X': 'Dead',
        'x': 'Dead',
        'K': 'Wakekill',
        'W': 'Waking',
        'P': 'Parked',
        'I': 'Idle'
    }

    if 'state' in proc_data:
        proc_data['state_pretty'] = state_map[proc_data['state']]

    for key, val in proc_data.items():
        try:
            proc_data[key] = int(val)
        except Exception:
            pass

    return proc_data


def parse(
    data: str,
    raw: bool = False,
    quiet: bool = False
) -> Dict:
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        Dictionary. Raw or processed structured data.
    """
    jc.utils.compatibility(__name__, info.compatible, quiet)
    jc.utils.input_type_check(data)

    raw_output: Dict = {}

    if jc.utils.has_data(data):

        line_re = re.compile(r'''
            ^(?P<pid>\d+)\s
            \((?P<comm>.+)\)\s
            (?P<state>\S)\s
            (?P<ppid>\d+)\s
            (?P<pgrp>\d+)\s
            (?P<session>\d+)\s
            (?P<tty_nr>\d+)\s
            (?P<tpg_id>-?\d+)\s
            (?P<flags>\d+)\s
            (?P<minflt>\d+)\s
            (?P<cminflt>\d+)\s
            (?P<majflt>\d+)\s
            (?P<cmajflt>\d+)\s
            (?P<utime>\d+)\s
            (?P<stime>\d+)\s
            (?P<cutime>\d+)\s
            (?P<cstime>\d+)\s
            (?P<priority>\d+)\s
            (?P<nice>\d+)\s
            (?P<num_threads>\d+)\s
            (?P<itrealvalue>\d+)\s
            (?P<starttime>\d+)\s
            (?P<vsize>\d+)\s
            (?P<rss>\d+)\s
            (?P<rsslim>\d+)\s
            (?P<startcode>\d+)\s
            (?P<endcode>\d+)\s
            (?P<startstack>\d+)\s
            (?P<kstkeep>\d+)\s
            (?P<kstkeip>\d+)\s
            (?P<signal>\d+)\s
            (?P<blocked>\d+)\s
            (?P<sigignore>\d+)\s
            (?P<sigcatch>\d+)\s
            (?P<wchan>\d+)\s
            (?P<nswap>\d+)\s
            (?P<cnswap>\d+)\s
            (?P<exit_signal>\d+)\s
            (?P<processor>\d+)\s
            (?P<rt_priority>\d+)\s
            (?P<policy>\d+)\s
            (?P<delayacct_blkio_ticks>\d+)\s
            (?P<guest_time>\d+)\s
            (?P<cguest_time>\d+)\s
            (?P<start_data>\d+)\s
            (?P<end_data>\d+)\s
            (?P<start_brk>\d+)\s
            (?P<arg_start>\d+)\s
            (?P<arg_end>\d+)\s
            (?P<env_start>\d+)\s
            (?P<env_end>\d+)\s
            (?P<exit_code>\d+)
        ''', re.VERBOSE | re.DOTALL
        )

        line_match = line_re.search(data)

        if line_match:
            raw_output = line_match.groupdict()

    return raw_output if raw else _process(raw_output)



import unittest
import os
import sys
sys.path.append(os.getcwd())
from tests import utils_for_test as test_utils
sys.path.pop()

class MyTests(unittest.TestCase):

    def test_path_nodata(self):
        """
        Test 'path' with no data
        """
        test_utils.run_no_data(self, __file__, {})

    def test_path_all_fixtures(self):
        """
        Test 'path' with various logs
        """
        test_utils.run_all_fixtures(self, __file__)


if __name__ == '__main__':
    unittest.main()



import pprint
import json
import re
import unittest
from typing import Optional

from jc.parsers.xrandr import (
    Device,
    Edid,
    _Line,
    LineType,
    ResolutionMode,
    Response,
    Screen,
    _device_pattern,
    _frequencies_pattern,
    _parse_device,
    _parse_resolution_mode,
    _parse_screen,
    _resolution_mode_pattern,
    _screen_pattern,
    parse,
)


class XrandrTests(unittest.TestCase):
    def test_xrandr_nodata(self):
        """
        Test 'xrandr' with no data
        """
        self.assertEqual(parse("", quiet=True), {"screens": []})

    def test_regexes(self):
        devices = [
            "HDMI1 connected (normal left inverted right x axis y axis)",
            "VIRTUAL1 disconnected (normal left inverted right x axis y axis)",
            "eDP1 connected primary 1920x1080+0+0 (normal left inverted right x axis y axis) 310mm x 170mm",
            "eDP-1 connected primary 1920x1080+0+0 (normal left inverted right x axis y axis) 309mm x 174mm",
            "HDMI-0 connected 2160x3840+3840+0 right (normal left inverted right x axis y axis) 609mm x 349mm",
            "LVDS-1 connected primary 1366x768+0+0 normal X axis (normal left inverted right x axis y axis) 609mm x 349mm",
            "VGA-1 connected 1280x1024+0+0 left X and Y axis (normal left inverted right x axis y axis) 609mm x 349mm",
        ]
        for device in devices:
            self.assertIsNotNone(re.match(_device_pattern, device))

        screens = [
            "Screen 0: minimum 8 x 8, current 1920 x 1080, maximum 32767 x 32767",
            "Screen 0: minimum 320 x 200, current 1920 x 1080, maximum 16384 x 16384",
        ]
        for screen in screens:
            self.assertIsNotNone(re.match(_screen_pattern, screen))

        modes = [
            "1920x1080     60.03*+  59.93",
            "1680x1050     59.88",
            "1400x1050     59.98",
            "1600x900      60.00    59.95    59.82",
            "1280x1024     60.02",
            "1400x900      59.96    59.88",
        ]
        for mode in modes:
            match = re.match(_resolution_mode_pattern, mode)
            self.assertIsNotNone(match)
            if match:
                rest = match.groupdict()["rest"]
                self.assertIsNotNone(re.match(_frequencies_pattern, rest))

    def test_line_categorize(self):
        base = "eDP-1 connected primary 1920x1080+0+0 (normal left inverted right x axis y axis) 309mm x 174mm"
        resolution_mode = "   320x240       60.05"
        prop_key = "	EDID:"
        prop_value = "		00ffffffffffff0006af3d5700000000"
        invalid = ""

        self.assertEqual(LineType.Device, _Line.categorize(base).t)
        self.assertEqual(LineType.ResolutionMode, _Line.categorize(resolution_mode).t)
        self.assertEqual(LineType.PropKey, _Line.categorize(prop_key).t)
        self.assertEqual(LineType.PropValue, _Line.categorize(prop_value).t)
        with self.assertRaises(Exception):
            _Line.categorize(invalid)

    def test_screens(self):
        sample = "Screen 0: minimum 8 x 8, current 1920 x 1080, maximum 32767 x 32767"
        line = _Line.categorize(sample)
        actual: Optional[Screen] = _parse_screen(line)
        self.assertIsNotNone(actual)

        expected = {
            "screen_number": 0,
            "minimum_width": 8,
            "minimum_height": 8,
            "current_width": 1920,
            "current_height": 1080,
            "maximum_width": 32767,
            "maximum_height": 32767,
        }
        if actual:
            for k, v in expected.items():
                self.assertEqual(v, actual[k], f"screens regex failed on {k}")

        sample = (
            "Screen 0: minimum 320 x 200, current 1920 x 1080, maximum 16384 x 16384"
        )
        line = _Line.categorize(sample)
        actual = _parse_screen(line)
        if actual:
            self.assertEqual(320, actual["minimum_width"])
        else:
            raise AssertionError("Screen should not be None")

    def test_device(self):
        # regex101 sample link for tests/edits https://regex101.com/r/3cHMv3/1
        sample = "eDP1 connected primary 1920x1080+0+0 left (normal left inverted right x axis y axis) 310mm x 170mm"
        line = _Line.categorize(sample)
        actual: Optional[Device] = _parse_device(line)

        expected = {
            "device_name": "eDP1",
            "is_connected": True,
            "is_primary": True,
            "resolution_width": 1920,
            "resolution_height": 1080,
            "offset_width": 0,
            "offset_height": 0,
            "dimension_width": 310,
            "dimension_height": 170,
            "rotation": "left",
        }

        self.assertIsNotNone(actual)

        if actual:
            for k, v in expected.items():
                self.assertEqual(v, actual[k], f"Devices regex failed on {k}")

        # with open("tests/fixtures/generic/xrandr_device.out", "r") as f:
        #     extended_sample = f.read().splitlines()

        # device = _parse_device(extended_sample)
        # if device:
        #     self.assertEqual(
        #         59.94, device["resolution_modes"][12]["frequencies"][4]["frequency"]
        #     )

    def test_device_with_reflect(self):
        sample = "VGA-1 connected primary 1920x1080+0+0 left X and Y axis (normal left inverted right x axis y axis) 310mm x 170mm"
        line = _Line.categorize(sample)
        actual: Optional[Device] = _parse_device(line)

        expected = {
            "device_name": "VGA-1",
            "is_connected": True,
            "is_primary": True,
            "resolution_width": 1920,
            "resolution_height": 1080,
            "offset_width": 0,
            "offset_height": 0,
            "dimension_width": 310,
            "dimension_height": 170,
            "rotation": "left",
            "reflection": "X and Y axis",
        }

        self.assertIsNotNone(actual)

        if actual:
            for k, v in expected.items():
                self.assertEqual(v, actual[k], f"Devices regex failed on {k}")

    def test_mode(self):
        sample_1 = "   1920x1080     60.03*+  59.93"
        expected = {
            "frequencies": [
                {"frequency": 60.03, "is_current": True, "is_preferred": True},
                {"frequency": 59.93, "is_current": False, "is_preferred": False},
            ],
            "resolution_width": 1920,
            "resolution_height": 1080,
            "is_high_resolution": False,
        }
        line = _Line.categorize(sample_1)
        actual: Optional[ResolutionMode] = _parse_resolution_mode(line)

        self.assertIsNotNone(actual)

        if actual:
            for k, v in expected.items():
                self.assertEqual(v, actual[k], f"mode regex failed on {k}")

        sample_2 = "   1920x1080i    60.00    50.00    59.94"
        line = _Line.categorize(sample_2)
        actual: Optional[ResolutionMode] = _parse_resolution_mode(line)
        self.assertIsNotNone(actual)
        if actual:
            self.assertEqual(True, actual["is_high_resolution"])
            self.assertEqual(50.0, actual["frequencies"][1]["frequency"])

    def test_complete_1(self):
        self.maxDiff = None
        with open("tests/fixtures/generic/xrandr.out", "r") as f:
            actual = parse(f.read(), quiet=True)

        with open('tests/fixtures/generic/xrandr.json', 'r') as f:
            reference = json.loads(f.read())

        self.assertEqual(1, len(actual["screens"]))
        self.assertEqual(
            18, len(actual["screens"][0]["devices"][0]["resolution_modes"])
        )
        self.assertEqual(actual, reference)

    def test_complete_2(self):
        with open("tests/fixtures/generic/xrandr_2.out", "r") as f:
            actual = parse(f.read(), quiet=True)

        with open('tests/fixtures/generic/xrandr_2.json', 'r') as f:
            reference = json.loads(f.read())

        self.assertEqual(1, len(actual["screens"]))
        self.assertEqual(
            38, len(actual["screens"][0]["devices"][0]["resolution_modes"])
        )
        self.assertEqual(actual, reference)

    def test_complete_3(self):
        with open("tests/fixtures/generic/xrandr_3.out", "r") as f:
            actual = parse(f.read(), quiet=True)

        with open('tests/fixtures/generic/xrandr_3.json', 'r') as f:
            reference = json.loads(f.read())

        self.assertEqual(1, len(actual["screens"]))
        self.assertEqual(
            2,
            len(actual["screens"][0]["devices"]),
        )
        self.assertEqual(actual, reference)

    def test_complete_4(self):
        with open("tests/fixtures/generic/xrandr_simple.out", "r") as f:
            actual = parse(f.read(), quiet=True)

        with open('tests/fixtures/generic/xrandr_simple.json', 'r') as f:
            reference = json.loads(f.read())

        self.assertEqual(1, len(actual["screens"]))
        self.assertEqual(2, len(actual["screens"][0]["devices"][0]["resolution_modes"]))
        self.assertEqual(actual, reference)

    def test_complete_5(self):
        with open("tests/fixtures/generic/xrandr_properties_1.out", "r") as f:
            actual = parse(f.read(), quiet=True)

        with open('tests/fixtures/generic/xrandr_properties_1.json', 'r') as f:
            reference = json.loads(f.read())

        self.assertEqual(1, len(actual["screens"]))
        self.assertEqual(
            38, len(actual["screens"][0]["devices"][0]["resolution_modes"])
        )
        self.assertEqual(actual, reference)

    # def test_model(self):
    #     asus_edid = [
    #         "   EDID: ",
    #         "         00ffffffffffff000469d41901010101",
    #         "         2011010308291a78ea8585a6574a9c26",
    #         "         125054bfef80714f8100810f81408180",
    #         "         9500950f01019a29a0d0518422305098",
    #         "         360098ff1000001c000000fd00374b1e",
    #         "         530f000a202020202020000000fc0041",
    #         "         535553205657313933530a20000000ff",
    #         "         0037384c383032313130370a20200077",
    #     ]
    #     asus_edid.reverse()

    #     expected = {
    #         "name": "ASUS VW193S",
    #         "product_id": "6612",
    #         "serial_number": "78L8021107",
    #     }

    #     actual: Optional[EdidModel] = _parse_model(asus_edid)
    #     self.assertIsNotNone(actual)

    #     if actual:
    #         for k, v in expected.items():
    #             self.assertEqual(v, actual[k], f"mode regex failed on {k}")

    #     generic_edid = [
    #         "   EDID: ",
    #         "         00ffffffffffff004ca3523100000000",
    #         "         0014010380221378eac8959e57549226",
    #         "         0f505400000001010101010101010101",
    #         "         010101010101381d56d4500016303020",
    #         "         250058c2100000190000000f00000000",
    #         "         000000000025d9066a00000000fe0053",
    #         "         414d53554e470a204ca34154000000fe",
    #         "         004c544e313536415432343430310018",
    #     ]
    #     generic_edid.reverse()

    #     expected = {
    #         "name": "Generic",
    #         "product_id": "12626",
    #         "serial_number": "0",
    #     }

    #     jc.parsers.xrandr.parse_state = {}
    #     actual: Optional[EdidModel] = _parse_model(generic_edid)
    #     self.assertIsNotNone(actual)

    #     if actual:
    #         for k, v in expected.items():
    #             self.assertEqual(v, actual[k], f"mode regex failed on {k}")

    #     empty_edid = [""]
    #     actual: Optional[EdidModel] = _parse_model(empty_edid)
    #     self.assertIsNone(actual)

    def test_issue_490(self):
        """test for issue 490: https://github.com/kellyjonbrazil/jc/issues/490"""
        data_in = """\
Screen 0: minimum 1024 x 600, current 1024 x 600, maximum 1024 x 600
default connected 1024x600+0+0 0mm x 0mm
   1024x600 0.00*
"""
        actual: Response = parse(data_in, quiet=True)
        self.maxDiff = None
        self.assertEqual(1024, actual["screens"][0]["devices"][0]["resolution_width"])

    def test_issue_525(self):
        self.maxDiff = None
        with open("tests/fixtures/generic/xrandr_issue_525.out", "r") as f:
            txt = f.read()
        actual = parse(txt, quiet=True)
        dp4 = actual["screens"][0]["devices"][0]["props"]["Broadcast RGB"][1]  # type: ignore
        # pprint.pprint(actual)
        self.assertEqual("supported: Automatic, Full, Limited 16:235", dp4)
        edp1_expected_keys = {
            "EDID",
            "EdidModel",
            "scaling mode",
            "Colorspace",
            "max bpc",
            "Broadcast RGB",
            "panel orientation",
            "link-status",
            "CTM",
            "CONNECTOR_ID",
            "non-desktop",
        }
        actual_keys = set(actual["screens"][0]["devices"][0]["props"].keys())
        self.assertSetEqual(edp1_expected_keys, actual_keys)
        expected_edid_model = {
            "name": "Generic",
            "product_id": "22333",
            "serial_number": "0",
        }
        self.assertDictEqual(
            expected_edid_model,
            actual["screens"][0]["devices"][0]["props"]["EdidModel"],  # type: ignore
        )

    def test_issue_549(self):
        """https://github.com/kellyjonbrazil/jc/issues/549"""
        with open("tests/fixtures/generic/xrandr_extra_hv_lines.out", "r") as f:
            actual = parse(f.read(), quiet=True)

        with open('tests/fixtures/generic/xrandr_extra_hv_lines.json', 'r') as f:
            reference = json.loads(f.read())

        self.assertEqual(actual, reference)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
import json
from typing import Dict
import jc.parsers.lspci

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):
    f_in: Dict = {}
    f_json: Dict = {}

    @classmethod
    def setUpClass(cls):
        fixtures = {
            'ubuntu_20_10_lspci_mmv': (
                'fixtures/ubuntu-20.10/lspci-mmv.out',
                'fixtures/ubuntu-20.10/lspci-mmv.json'),
            'ubuntu_20_10_lspci_nmmv': (
                'fixtures/ubuntu-20.10/lspci-nmmv.out',
                'fixtures/ubuntu-20.10/lspci-nmmv.json'),
            'ubuntu_20_10_lspci_nnmmv': (
                'fixtures/ubuntu-20.10/lspci-nnmmv.out',
                'fixtures/ubuntu-20.10/lspci-nnmmv.json')
        }

        for file, filepaths in fixtures.items():
            with open(os.path.join(THIS_DIR, filepaths[0]), 'r', encoding='utf-8') as a, \
                 open(os.path.join(THIS_DIR, filepaths[1]), 'r', encoding='utf-8') as b:
                cls.f_in[file] = a.read()
                cls.f_json[file] = json.loads(b.read())


    def test_lspci_nodata(self):
        """
        Test 'lspci' with no data
        """
        self.assertEqual(jc.parsers.lspci.parse('', quiet=True), [])

    def test_lspci_mmv_ubuntu_20_10(self):
        """
        Test 'lspci -mmv' on Ubuntu 20.10
        """
        self.assertEqual(jc.parsers.lspci.parse(self.f_in['ubuntu_20_10_lspci_mmv'], quiet=True),
                                                self.f_json['ubuntu_20_10_lspci_mmv'])

    def test_lspci_nmmv_ubuntu_20_10(self):
        """
        Test 'lspci -nmmv' on Ubuntu 20.10
        """
        self.assertEqual(jc.parsers.lspci.parse(self.f_in['ubuntu_20_10_lspci_nmmv'], quiet=True),
                                                self.f_json['ubuntu_20_10_lspci_nmmv'])

    def test_lspci_nnmmv_ubuntu_20_10(self):
        """
        Test 'lspci -nnmmv' on Ubuntu 20.10
        """
        self.assertEqual(jc.parsers.lspci.parse(self.f_in['ubuntu_20_10_lspci_nnmmv'], quiet=True),
                                                self.f_json['ubuntu_20_10_lspci_nnmmv'])


if __name__ == '__main__':
    unittest.main()



import sys
import unittest
import jc.utils


class MyTests(unittest.TestCase):

    def test_utils_timestamp(self):
        # naive timestamps created in PDT
        datetime_map = {
            # C locale format conversion, or date cli command in C locale with non-UTC tz
            'Tue Mar 23 16:12:11 2021': {'string': 'Tue Mar 23 16:12:11 2021', 'format': 1000, 'naive': 1616541131, 'utc': None},
            'Tue Mar 23 16:12:11 IST 2021': {'string': 'Tue Mar 23 16:12:11 IST 2021', 'format': 1000, 'naive': 1616541131, 'utc': None},
            # Git date output
            'Thu Mar 5 09:17:40 2020 -0800': {'string': 'Thu Mar 5 09:17:40 2020 -0800', 'format': 1100, 'naive': 1583428660, 'utc': None},
            # ISO Format with UTC (found in syslog 5424)
            '2003-10-11T22:14:15.003Z': {'string': '2003-10-11T22:14:15.003Z', 'format': 1300, 'naive': 1065935655, 'utc': 1065910455},
            # ISO Format without TZ (found in syslog 5424)
            '2003-10-11T22:14:15.003': {'string': '2003-10-11T22:14:15.003', 'format': 1310, 'naive': 1065935655, 'utc': None},
            # CEF Format with UTC
            'Nov 08 2022 12:30:00.111 UTC': {'string': 'Nov 08 2022 12:30:00.111 UTC', 'format': 1400, 'naive': 1667939400, 'utc': 1667910600},
            # CEF Format without TZ
            'Nov 08 2022 12:30:00.111': {'string': 'Nov 08 2022 12:30:00.111', 'format': 1410, 'naive': 1667939400, 'utc': None},
            # CEF Format with UTC without microseconds
            'Nov 08 2022 12:30:00 UTC': {'string': 'Nov 08 2022 12:30:00 UTC', 'format': 1420, 'naive': 1667939400, 'utc': 1667910600},
            # CEF Format without TZ or microseconds
            'Nov 08 2022 12:30:00': {'string': 'Nov 08 2022 12:30:00', 'format': 1430, 'naive': 1667939400, 'utc': None},
            # en_US.UTF-8 local format (found in who cli output)
            '2021-03-23 00:14': {'string': '2021-03-23 00:14', 'format': 1500, 'naive': 1616483640, 'utc': None},
            # Windows english format (found in dir cli output)
            '12/07/2019 02:09 AM': {'string': '12/07/2019 02:09 AM', 'format': 1600, 'naive': 1575713340, 'utc': None},
            # Windows english format wint non-UTC tz (found in systeminfo cli output)
            '3/22/2021, 1:15:51 PM (UTC-0600)': {'string': '3/22/2021, 1:15:51 PM (UTC-0600)', 'format': 1700, 'naive': 1616444151, 'utc': None},
            # Windows english format with UTC tz (found in systeminfo cli output)
            '3/22/2021, 1:15:51 PM (UTC)': {'string': '3/22/2021, 1:15:51 PM (UTC)', 'format': 1705, 'naive': 1616444151, 'utc': 1616418951},
            # Windows english format with UTC tz in long-form (found in systeminfo cli output)
            '3/22/2021, 1:15:51 PM (Coordinated Universal Time)': {'string': '3/22/2021, 1:15:51 PM (Coordinated Universal Time)', 'format': 1705, 'naive': 1616444151, 'utc': 1616418951},
            # Windows english format with UTC tz (found in systeminfo cli output)
            '3/22/2021, 1:15:51 PM (UTC+0000)': {'string': '3/22/2021, 1:15:51 PM (UTC+0000)', 'format': 1710, 'naive': 1616444151, 'utc': 1616418951},
            # Windows ipconfig cli output format
            'Thursday, June 22, 2023 10:39:04 AM': {'string': 'Thursday, June 22, 2023 10:39:04 AM', 'format': 1720, 'naive': 1687455544, 'utc': None},
            # Google Big Table format with no timezone:
            '2000/01/01-01:00:00.000000': {'string': '2000/01/01-01:00:00.000000', 'format': 1750, 'naive': 946717200, 'utc': None},
            # Google Big Table format with timezone:
            '2000/01/01-01:00:00.000000+00:00': {'string': '2000/01/01-01:00:00.000000+00:00', 'format': 1755, 'naive': 946717200, 'utc': 946688400},
            # certbot format with timezone:
            '2023-05-11 01:33:10+00:00': {'string': '2023-05-11 01:33:10+00:00', 'format': 1760, 'naive': 1683793990, 'utc': 1683768790},
            # Common Log Format
            '10/Oct/2000:13:55:36 -0700': {'string': '10/Oct/2000:13:55:36 -0700', 'format': 1800, 'naive': 971211336, 'utc': None},
            '10/Oct/2000:13:55:36 -0000': {'string': '10/Oct/2000:13:55:36 -0000', 'format': 1800, 'naive': 971211336, 'utc': 971186136},
            # en_US.UTF-8 local format (found in upower cli output)
            'Tue 23 Mar 2021 04:12:11 PM UTC': {'string': 'Tue 23 Mar 2021 04:12:11 PM UTC', 'format': 2000, 'naive': 1616541131, 'utc': 1616515931},
            # en_US.UTF-8 local format with non-UTC tz (found in upower cli output)
            'Tue 23 Mar 2021 04:12:11 PM IST': {'string': 'Tue 23 Mar 2021 04:12:11 PM IST', 'format': 3000, 'naive': 1616541131, 'utc': None},
            # HTTP header time format (always GMT so assume UTC)
            'Wed, 31 Jan 2024 00:39:28 GMT': {'string': 'Wed, 31 Jan 2024 00:39:28 GMT', 'format': 3500, 'naive': 1706690368, 'utc': 1706661568},
            # European local format (found in upower cli output)
            'Tuesday 01 October 2019 12:50:41 PM UTC': {'string': 'Tuesday 01 October 2019 12:50:41 PM UTC', 'format': 4000, 'naive': 1569959441, 'utc': 1569934241},
            # European local format with non-UTC tz (found in upower cli output)
            'Tuesday 01 October 2019 12:50:41 PM IST': {'string': 'Tuesday 01 October 2019 12:50:41 PM IST', 'format': 5000, 'naive': 1569959441, 'utc': None},
            # date cli command in en_US.UTF-8 format
            'Wed Mar 24 06:16:19 PM UTC 2021': {'string': 'Wed Mar 24 06:16:19 PM UTC 2021', 'format': 6000, 'naive': 1616634979, 'utc': 1616609779},
            # date cli command in C locale format
            'Wed Mar 24 11:11:30 UTC 2021': {'string': 'Wed Mar 24 11:11:30 UTC 2021', 'format': 7000, 'naive': 1616609490, 'utc': 1616584290},
            # C locale format (found in stat cli output - OSX)
            'Mar 29 11:49:05 2021': {'string': 'Mar 29 11:49:05 2021', 'format': 7100, 'naive': 1617043745, 'utc': None},
            # C local format (found in stat cli output - linux) non-UTC tz
            '2019-08-13 18:13:43.555604315 -0400': {'string': '2019-08-13 18:13:43.555604315 -0400', 'format': 7200, 'naive': 1565745223, 'utc': None},
            # C local format (found in stat cli output - linux) UTC
            '2019-08-13 18:13:43.555604315 -0000': {'string': '2019-08-13 18:13:43.555604315 -0000', 'format': 7200, 'naive': 1565745223, 'utc': 1565720023},
            # C locale format with non-UTC tz (found in modified vmstat cli output)
            '2021-09-16 20:32:28 PDT': {'string': '2021-09-16 20:32:28 PDT', 'format': 7250, 'naive': 1631849548, 'utc': None},
            # C locale format (found in modified vmstat cli output)
            '2021-09-16 20:32:28 UTC': {'string': '2021-09-16 20:32:28 UTC', 'format': 7255, 'naive': 1631849548, 'utc': 1631824348},
            # C locale format (found in timedatectl cli output)
            'Wed 2020-03-11 00:53:21 UTC': {'string': 'Wed 2020-03-11 00:53:21 UTC', 'format': 7300, 'naive': 1583913201, 'utc': 1583888001},
            # test with None input
            None: {'string': None, 'format': None, 'naive': None, 'utc': None}
        }

        # fixup for change in behavior after python 3.6:
        # Changed in version 3.7: When the %z directive is provided to the strptime() method,
        # the UTC offsets can have a colon as a separator between hours, minutes and seconds.
        # For example, '+01:00:00' will be parsed as an offset of one hour. In addition,
        # providing 'Z' is identical to '+00:00'.
        if sys.version_info < (3, 7, 0):
            del datetime_map['2000/01/01-01:00:00.000000+00:00']

        for input_string, expected_output in datetime_map.items():
            ts = jc.utils.timestamp(input_string)
            ts_dict = {
                'string': ts.string,
                'format': ts.format,
                'naive': ts.naive,
                'utc': ts.utc
            }

            try:
                self.assertEqual(ts_dict, expected_output)
            except AssertionError:
                # fixup for behavior change in python 3.15
                format_changes = {7250: 1760}
                if sys.version_info >= (3, 15, 0) and expected_output['format'] in format_changes:
                    expected_output['format'] = format_changes[expected_output['format']]
                    self.assertEqual(ts_dict, expected_output)
                else:
                    raise

    def test_utils_convert_to_int(self):
        io_map = {
            None: None,
            True: 1,
            False: 0,
            '': None,
            '0': 0,
            '1': 1,
            '-1': -1,
            '0.0': 0,
            '0.1': 0,
            '0.6': 0,
            '-0.1': 0,
            '-0.6': 0,
            0: 0,
            1: 1,
            -1: -1,
            0.0: 0,
            0.1: 0,
            0.6: 0,
            -0.1: 0,
            -0.6: 0
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_to_int(input_string), expected_output)

    def test_utils_convert_to_float(self):
        io_map = {
            None: None,
            True: 1.0,
            False: 0.0,
            '': None,
            '0': 0.0,
            '1': 1.0,
            '-1': -1.0,
            '0.0': 0.0,
            '0.1': 0.1,
            '0.6': 0.6,
            '-0.1': -0.1,
            '-0.6': -0.6,
            0: 0.0,
            1: 1.0,
            -1: -1.0,
            0.0: 0.0,
            0.1: 0.1,
            0.6: 0.6,
            -0.1: -0.1,
            -0.6: -0.6
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_to_float(input_string), expected_output)

    def test_utils_convert_to_bool(self):
        io_map = {
            None: False,
            True: True,
            False: False,
            '': False,
            '0': False,
            '1': True,
            '-1': True,
            '0.0': False,
            '0.1': True,
            '-0.1': True,
            '*': True,
            'true': True,
            'True': True,
            'false': False,
            'False': False,
            'Y': True,
            'y': True,
            'Yes': True,
            'n': False,
            'N': False,
            'No': False,
            0: False,
            1: True,
            -1: True,
            0.0: False,
            0.1: True,
            -0.1: True,
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_to_bool(input_string), expected_output)


    def test_utils_convert_size_to_int(self):
        io_map = {
            '42': 42,
            '13b': 13,
            '5 bytes': 5,
            '1 KB': 1000,
            '1 kilobyte': 1000,
            '1 KiB': 1024,
            '1.5 GB': 1500000000
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_size_to_int(input_string), expected_output)


    def test_utils_convert_size_to_int_binary_true(self):
        io_map = {
            '1 KB': 1024,
            '1.5 GB': 1610612736
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_size_to_int(input_string, binary=True), expected_output)


    def test_utils_convert_size_to_int_posix_mode(self):
        io_map = {
            '1 K': 1024,
            '1 KiB': 1024,
            '1 KB': 1000,
            '1.5 G': 1610612736,
            '1.5 GiB': 1610612736,
            '1.5 GB': 1500000000
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_size_to_int(input_string, posix_mode=True), expected_output)


    def test_utils_convert_size_to_int_decimal_bias(self):
        io_map = {
            '1 K': 1000,
            '1 Ki': 1000,
            '1 KiB': 1024,
            '1.5 G': 1500000000,
            '1.5 Gi': 1500000000,
            '1.5 GiB': 1610612736
        }

        for input_string, expected_output in io_map.items():
            self.assertEqual(jc.utils.convert_size_to_int(input_string, decimal_bias=True), expected_output)


    def test_utils_has_data_nodata(self):
        self.assertFalse(jc.utils.has_data('     \n      '))


    def test_utils_has_data_withdata(self):
        self.assertTrue(jc.utils.has_data('     \n  abcd    \n    '))


    def test_utils_input_type_check_wrong(self):
        self.assertRaises(TypeError, jc.utils.input_type_check, ['abc'])


    def test_utils_input_type_check_correct(self):
        self.assertEqual(jc.utils.input_type_check('abc'), None)


    def test_utils_line_slice_string_positive_slice(self):
        data = '''line1
line2
line3
line4
'''
        expected = 'line2\nline3'
        self.assertEqual(jc.utils.line_slice(data, 1, 3), expected)


    def test_utils_line_slice_string_negative_slice(self):
        data = '''line1
line2
line3
line4
'''
        expected = 'line2\nline3'
        self.assertEqual(jc.utils.line_slice(data, 1, -1), expected)


    def test_utils_line_slice_iter_positive_slice(self):
        data = [
            'line1',
            'line2',
            'line3',
            'line4'
        ]
        expected = ['line2', 'line3']
        self.assertEqual(list(jc.utils.line_slice(data, 1, 3)), expected)


    def test_utils_line_slice_iter_negative_slice(self):
        data = [
            'line1',
            'line2',
            'line3',
            'line4'
        ]
        expected = ['line2', 'line3']
        self.assertEqual(list(jc.utils.line_slice(data, 1, -1)), expected)

    def test_utils_line_slice_string_blank_lines(self):
        data = '''line1
line2

line4
line5
'''
        expected = 'line2\n\nline4'
        self.assertEqual(jc.utils.line_slice(data, 1, 4), expected)

    def test_utils_line_slice_iter_positive_blank_lines(self):
        data = [
            'line1',
            'line2',
            '',
            'line4',
            'line5'
        ]
        expected = ['line2', '', 'line4']
        self.assertEqual(list(jc.utils.line_slice(data, 1, 4)), expected)

    def test_remove_quotes(self):
        for char in ["'", '"']:
            with self.subTest(f'Quote character: {char}'):
                data = f'{char}this is a test{char}'
                expected = 'this is a test'
                self.assertEqual(jc.utils.remove_quotes(data), expected)

    def test_normalize_key(self):
        io_map = {
            'This is @ crazy  Key!!': 'this_is_crazy_key',
            'Simple': 'simple',
            'CamelCase': 'camelcase',
            '^Complex-Key*': '_complex_key'
        }

        for data, expected in io_map.items():
            with self.subTest(f'Original key: {data}'):
                self.assertEqual(jc.utils.normalize_key(data), expected)


    # need to mock shutil.get_terminal_size().columns or add a column parameter to test
    # def test_utils_warning_message(self):
    #     msg = [
    #         'this is a long first line that will be wrapped yada yada yada yada yada yada yada.',
    #         'this is a second long line that will be wrapped yada yada yada yada yada yada yada yada yada.',
    #         'this is a third long line that will be wrapped yada yada yada yada yada yada yada yada yada.'
    #     ]

    #     expected = '''jc:  Warning - this is a long first line that will be wrapped yada yada yada
    #            yada yada yada yada.
    #            this is a second long line that will be wrapped yada yada yada
    #                yada yada yada yada yada yada.
    #            this is a third long line that will be wrapped yada yada yada
    #                yada yada yada yada yada yada.'''

    #     f = io.StringIO()
    #     with contextlib.redirect_stderr(f):
    #         jc.utils.warning_message(msg)

    #     self.assertEqual(f.getvalue(), expected + '\n')



if __name__ == '__main__':
    unittest.main()



import os
import unittest
import json
from typing import Dict
import jc.parsers.proc
from jc.exceptions import ParseError

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):
    """
    These tests replicate the individual /proc file module tests, but the purpose
    is different. These tests ensure the file signature matching engine in the
    /proc parser module sends the user input to the correct parser.

    Signature match regex can be order dependent, so these tests make sure we
    don't break anything when adding/removing/re-ordering the signature regexes.
    """
    f_in: Dict = {}
    f_json: Dict = {}

    @classmethod
    def setUpClass(cls):
        fixtures = {
            'proc_buddyinfo': (
                'fixtures/linux-proc/buddyinfo',
                'fixtures/linux-proc/buddyinfo.json'),
            'proc_consoles': (
                'fixtures/linux-proc/consoles',
                'fixtures/linux-proc/consoles.json'),
            'proc_consoles2': (
                'fixtures/linux-proc/consoles2',
                'fixtures/linux-proc/consoles2.json'),
            'proc_cpuinfo': (
                'fixtures/linux-proc/cpuinfo',
                'fixtures/linux-proc/cpuinfo.json'),
            'proc_cpuinfo2': (
                'fixtures/linux-proc/cpuinfo2',
                'fixtures/linux-proc/cpuinfo2.json'),
            'proc_crypto': (
                'fixtures/linux-proc/crypto',
                'fixtures/linux-proc/crypto.json'),
            'proc_devices': (
                'fixtures/linux-proc/devices',
                'fixtures/linux-proc/devices.json'),
            'proc_diskstats': (
                'fixtures/linux-proc/diskstats',
                'fixtures/linux-proc/diskstats.json'),
            'proc_filesystems': (
                'fixtures/linux-proc/filesystems',
                'fixtures/linux-proc/filesystems.json'),
            'proc_interrupts': (
                'fixtures/linux-proc/interrupts',
                'fixtures/linux-proc/interrupts.json'),
            'proc_iomem': (
                'fixtures/linux-proc/iomem',
                'fixtures/linux-proc/iomem.json'),
            'proc_ioports': (
                'fixtures/linux-proc/ioports',
                'fixtures/linux-proc/ioports.json'),
            'proc_loadavg': (
                'fixtures/linux-proc/loadavg',
                'fixtures/linux-proc/loadavg.json'),
            'proc_locks': (
                'fixtures/linux-proc/locks',
                'fixtures/linux-proc/locks.json'),
            'proc_meminfo': (
                'fixtures/linux-proc/meminfo',
                'fixtures/linux-proc/meminfo.json'),
            'proc_modules': (
                'fixtures/linux-proc/modules',
                'fixtures/linux-proc/modules.json'),
            'proc_mtrr': (
                'fixtures/linux-proc/mtrr',
                'fixtures/linux-proc/mtrr.json'),
            'proc_pagetypeinfo': (
                'fixtures/linux-proc/pagetypeinfo',
                'fixtures/linux-proc/pagetypeinfo.json'),
            'proc_partitions': (
                'fixtures/linux-proc/partitions',
                'fixtures/linux-proc/partitions.json'),
            'proc_slabinfo': (
                'fixtures/linux-proc/slabinfo',
                'fixtures/linux-proc/slabinfo.json'),
            'proc_softirqs': (
                'fixtures/linux-proc/softirqs',
                'fixtures/linux-proc/softirqs.json'),
            'proc_stat': (
                'fixtures/linux-proc/stat',
                'fixtures/linux-proc/stat.json'),
            'proc_stat2': (
                'fixtures/linux-proc/stat2',
                'fixtures/linux-proc/stat2.json'),
            'proc_swaps': (
                'fixtures/linux-proc/swaps',
                'fixtures/linux-proc/swaps.json'),
            'proc_uptime': (
                'fixtures/linux-proc/uptime',
                'fixtures/linux-proc/uptime.json'),
            'proc_version': (
                'fixtures/linux-proc/version',
                'fixtures/linux-proc/version.json'),
            'proc_version2': (
                'fixtures/linux-proc/version2',
                'fixtures/linux-proc/version2.json'),
            'proc_version3': (
                'fixtures/linux-proc/version3',
                'fixtures/linux-proc/version3.json'),
            'proc_vmallocinfo': (
                'fixtures/linux-proc/vmallocinfo',
                'fixtures/linux-proc/vmallocinfo.json'),
            'proc_vmstat': (
                'fixtures/linux-proc/vmstat',
                'fixtures/linux-proc/vmstat.json'),
            'proc_zoneinfo': (
                'fixtures/linux-proc/zoneinfo',
                'fixtures/linux-proc/zoneinfo.json'),
            'proc_zoneinfo2': (
                'fixtures/linux-proc/zoneinfo2',
                'fixtures/linux-proc/zoneinfo2.json'),

            'proc_driver_rtc': (
                'fixtures/linux-proc/driver_rtc',
                'fixtures/linux-proc/driver_rtc.json'),

            'proc_net_arp': (
                'fixtures/linux-proc/net_arp',
                'fixtures/linux-proc/net_arp.json'),
            'proc_net_dev_mcast': (
                'fixtures/linux-proc/net_dev_mcast',
                'fixtures/linux-proc/net_dev_mcast.json'),
            'proc_net_dev': (
                'fixtures/linux-proc/net_dev',
                'fixtures/linux-proc/net_dev.json'),
            'proc_net_if_inet6': (
                'fixtures/linux-proc/net_if_inet6',
                'fixtures/linux-proc/net_if_inet6.json'),
            'proc_net_igmp': (
                'fixtures/linux-proc/net_igmp',
                'fixtures/linux-proc/net_igmp.json'),
            'proc_net_igmp_more': (
                'fixtures/linux-proc/net_igmp_more',
                'fixtures/linux-proc/net_igmp_more.json'),
            'proc_net_igmp6': (
                'fixtures/linux-proc/net_igmp6',
                'fixtures/linux-proc/net_igmp6.json'),
            'proc_net_ipv6_route': (
                'fixtures/linux-proc/net_ipv6_route',
                'fixtures/linux-proc/net_ipv6_route.json'),
            'proc_net_netlink': (
                'fixtures/linux-proc/net_netlink',
                'fixtures/linux-proc/net_netlink.json'),
            'proc_net_netstat': (
                'fixtures/linux-proc/net_netstat',
                'fixtures/linux-proc/net_netstat.json'),
            'proc_net_packet': (
                'fixtures/linux-proc/net_packet',
                'fixtures/linux-proc/net_packet.json'),
            'proc_net_protocols': (
                'fixtures/linux-proc/net_protocols',
                'fixtures/linux-proc/net_protocols.json'),
            'proc_net_route': (
                'fixtures/linux-proc/net_route',
                'fixtures/linux-proc/net_route.json'),
            'proc_net_unix': (
                'fixtures/linux-proc/net_unix',
                'fixtures/linux-proc/net_unix.json'),

            'proc_pid_fdinfo': (
                'fixtures/linux-proc/pid_fdinfo',
                'fixtures/linux-proc/pid_fdinfo.json'),
            'proc_pid_fdinfo_dma': (
                'fixtures/linux-proc/pid_fdinfo_dma',
                'fixtures/linux-proc/pid_fdinfo_dma.json'),
            'proc_pid_fdinfo_epoll': (
                'fixtures/linux-proc/pid_fdinfo_epoll',
                'fixtures/linux-proc/pid_fdinfo_epoll.json'),
            'proc_pid_fdinfo_fanotify': (
                'fixtures/linux-proc/pid_fdinfo_fanotify',
                'fixtures/linux-proc/pid_fdinfo_fanotify.json'),
            'proc_pid_fdinfo_inotify': (
                'fixtures/linux-proc/pid_fdinfo_inotify',
                'fixtures/linux-proc/pid_fdinfo_inotify.json'),
            'proc_pid_fdinfo_timerfd': (
                'fixtures/linux-proc/pid_fdinfo_timerfd',
                'fixtures/linux-proc/pid_fdinfo_timerfd.json'),
            'proc_pid_io': (
                'fixtures/linux-proc/pid_io',
                'fixtures/linux-proc/pid_io.json'),
            'proc_pid_maps': (
                'fixtures/linux-proc/pid_maps',
                'fixtures/linux-proc/pid_maps.json'),
            'proc_pid_mountinfo': (
                'fixtures/linux-proc/pid_mountinfo',
                'fixtures/linux-proc/pid_mountinfo.json'),
            'proc_pid_numa_maps': (
                'fixtures/linux-proc/pid_numa_maps',
                'fixtures/linux-proc/pid_numa_maps.json'),
            'proc_pid_smaps': (
                'fixtures/linux-proc/pid_smaps',
                'fixtures/linux-proc/pid_smaps.json'),
            'proc_pid_stat': (
                'fixtures/linux-proc/pid_stat',
                'fixtures/linux-proc/pid_stat.json'),
            'pid_stat_w_space_and_nl_in_comm': (
                'fixtures/linux-proc/pid_stat_w_space_and_nl_in_comm',
                'fixtures/linux-proc/pid_stat_w_space_and_nl_in_comm.json'),
            'proc_pid_stat_hack': (
                'fixtures/linux-proc/pid_stat_hack',
                'fixtures/linux-proc/pid_stat_hack.json'),
            'proc_pid_statm': (
                'fixtures/linux-proc/pid_statm',
                'fixtures/linux-proc/pid_statm.json'),
            'proc_pid_status': (
                'fixtures/linux-proc/pid_status',
                'fixtures/linux-proc/pid_status.json')
        }

        for file, filepaths in fixtures.items():
            with open(os.path.join(THIS_DIR, filepaths[0]), 'r', encoding='utf-8') as a, \
                 open(os.path.join(THIS_DIR, filepaths[1]), 'r', encoding='utf-8') as b:
                cls.f_in[file] = a.read()
                cls.f_json[file] = json.loads(b.read())


    def test_proc_nodata(self):
        """
        Test proc parser with no data
        """
        self.assertRaises(ParseError, jc.parsers.proc.parse, '', quiet=True)

    def test_proc_file_signature_detection(self):
        """
        Test proc parser file signature detection
        """
        for in_, expected in zip(self.f_in.keys(), self.f_json.keys()):
            self.assertEqual(jc.parsers.proc.parse(self.f_in[in_], quiet=True),
                                                   self.f_json[expected])


if __name__ == '__main__':
    unittest.main()



r"""jc - JSON Convert `udevadm info` command output parser

Usage (cli):

    $ udevadm info --query=all /dev/sda | jc --udevadm

or

    $ jc udevadm info --query=all /dev/sda

Usage (module):

    import jc
    result = jc.parse('udevadm', udevadm_command_output)

Schema:

    {
      "P":                    string,
      "N":                    string,
      "L":                    integer,
      "S": [
                              string
      ],
      "E": {
        "<key>":              string
      }
    }


Examples:

    $ udevadm info --query=all /dev/sda | jc --udevadm -p
    {
      "P": "/devices/pci0000:00/0000:00:10.0/host32/target32:0:0/32:0:0:0/block/sda",
      "N": "sda",
      "L": 0,
      "S": [
        "disk/by-path/pci-0000:00:10.0-scsi-0:0:0:0"
      ],
      "E": {
        "DEVPATH": "/devices/pci0000:00/0000:00:10.0/host32/target32:0:0/32:0:0:0/block/sda",
        "DEVNAME": "/dev/sda",
        "DEVTYPE": "disk",
        "MAJOR": "8",
        "MINOR": "0",
        "SUBSYSTEM": "block",
        "USEC_INITIALIZED": "6100111",
        "SCSI_TPGS": "0",
        "SCSI_TYPE": "disk",
        "SCSI_VENDOR": "VMware,",
        "SCSI_VENDOR_ENC": "VMware,\\x20",
        "SCSI_MODEL": "VMware_Virtual_S",
        "SCSI_MODEL_ENC": "VMware\\x20Virtual\\x20S",
        "SCSI_REVISION": "1.0",
        "ID_SCSI": "1",
        "ID_VENDOR": "VMware_",
        "ID_VENDOR_ENC": "VMware\\x2c\\x20",
        "ID_MODEL": "VMware_Virtual_S",
        "ID_MODEL_ENC": "VMware\\x20Virtual\\x20S",
        "ID_REVISION": "1.0",
        "ID_TYPE": "disk",
        "MPATH_SBIN_PATH": "/sbin",
        "ID_BUS": "scsi",
        "ID_PATH": "pci-0000:00:10.0-scsi-0:0:0:0",
        "ID_PATH_TAG": "pci-0000_00_10_0-scsi-0_0_0_0",
        "ID_PART_TABLE_UUID": "a5bd0c01-4210-46f2-b558-5c11c209a8f7",
        "ID_PART_TABLE_TYPE": "gpt",
        "DEVLINKS": "/dev/disk/by-path/pci-0000:00:10.0-scsi-0:0:0:0",
        "TAGS": ":systemd:"
      }
    }

    $ udevadm info --query=all /dev/sda | jc --udevadm -p -r
    {
      "P": "/devices/pci0000:00/0000:00:10.0/host32/target32:0:0/32:0:0:0/block/sda",
      "N": "sda",
      "L": "0",
      "S": [
        "disk/by-path/pci-0000:00:10.0-scsi-0:0:0:0"
      ],
      "E": {
        "DEVPATH": "/devices/pci0000:00/0000:00:10.0/host32/target32:0:0/32:0:0:0/block/sda",
        "DEVNAME": "/dev/sda",
        "DEVTYPE": "disk",
        "MAJOR": "8",
        "MINOR": "0",
        "SUBSYSTEM": "block",
        "USEC_INITIALIZED": "6100111",
        "SCSI_TPGS": "0",
        "SCSI_TYPE": "disk",
        "SCSI_VENDOR": "VMware,",
        "SCSI_VENDOR_ENC": "VMware,\\x20",
        "SCSI_MODEL": "VMware_Virtual_S",
        "SCSI_MODEL_ENC": "VMware\\x20Virtual\\x20S",
        "SCSI_REVISION": "1.0",
        "ID_SCSI": "1",
        "ID_VENDOR": "VMware_",
        "ID_VENDOR_ENC": "VMware\\x2c\\x20",
        "ID_MODEL": "VMware_Virtual_S",
        "ID_MODEL_ENC": "VMware\\x20Virtual\\x20S",
        "ID_REVISION": "1.0",
        "ID_TYPE": "disk",
        "MPATH_SBIN_PATH": "/sbin",
        "ID_BUS": "scsi",
        "ID_PATH": "pci-0000:00:10.0-scsi-0:0:0:0",
        "ID_PATH_TAG": "pci-0000_00_10_0-scsi-0_0_0_0",
        "ID_PART_TABLE_UUID": "a5bd0c01-4210-46f2-b558-5c11c209a8f7",
        "ID_PART_TABLE_TYPE": "gpt",
        "DEVLINKS": "/dev/disk/by-path/pci-0000:00:10.0-scsi-0:0:0:0",
        "TAGS": ":systemd:"
      }
    }
"""
from typing import List, Dict
from jc.jc_types import JSONDictType
import jc.utils


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.0'
    description = '`udevadm info` command parser'
    author = 'Kelly Brazil'
    author_email = 'kellyjonbrazil@gmail.com'
    compatible = ['linux']
    magic_commands = ['udevadm info']
    tags = ['command']


__version__ = info.version


def _process(proc_data: JSONDictType) -> JSONDictType:
    """
    Final processing to conform to the schema.

    Parameters:

        proc_data:   (List of Dictionaries) raw structured data to process

    Returns:

        List of Dictionaries. Structured to conform to the schema.
    """
    if 'L' in proc_data:
        proc_data['L'] = int(proc_data['L'])

    return proc_data


def parse(
    data: str,
    raw: bool = False,
    quiet: bool = False
) -> JSONDictType:
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        Dictionary. Raw or processed structured data.
    """
    jc.utils.compatibility(__name__, info.compatible, quiet)
    jc.utils.input_type_check(data)

    raw_output: Dict = {}
    s_list: List = []
    e_list: List = []

    if jc.utils.has_data(data):
        for line in filter(None, data.splitlines()):
            prefix, value = line.split(maxsplit=1)

            if prefix == 'P:':
                raw_output['P'] = value
                continue

            if prefix == 'S:':
                s_list.append(value)
                continue

            if prefix == 'E:':
                e_list.append(value)
                continue

            raw_output[prefix[:-1]] = value

    if s_list:
        raw_output['S'] = s_list

    if e_list:
        raw_output['E'] = {}
        for item in e_list:
            k, v = item.split('=')
            raw_output['E'][k] = v

    return raw_output if raw else _process(raw_output)



import os
import unittest
import json
import jc.parsers.blkid

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):

    # input
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid.out'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-sda2.out'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_sda2 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-sda2.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_sda2 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-ip-udev.out'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_ip_udev = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-ip-udev.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_ip_udev = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-ip-multi.out'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_ip_multi = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-ip-multi.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_ip_multi = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-ip-udev-multi.out'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_ip_udev_multi = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-ip-udev-multi.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_ip_udev_multi = f.read()

    # output
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid.json'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-sda2.json'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_sda2_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-sda2.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_sda2_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-ip-udev.json'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_ip_udev_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-ip-udev.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_ip_udev_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-ip-multi.json'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_ip_multi_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-ip-multi.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_ip_multi_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/blkid-ip-udev-multi.json'), 'r', encoding='utf-8') as f:
        centos_7_7_blkid_ip_udev_multi_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/blkid-ip-udev-multi.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_blkid_ip_udev_multi_json = json.loads(f.read())

    def test_blkid_nodata(self):
        """
        Test 'blkid' with no data
        """
        self.assertEqual(jc.parsers.blkid.parse('', quiet=True), [])

    def test_blkid_centos_7_7(self):
        """
        Test 'blkid' on Centos 7.7
        """
        self.assertEqual(jc.parsers.blkid.parse(self.centos_7_7_blkid, quiet=True), self.centos_7_7_blkid_json)

    def test_blkid_ubuntu_18_4(self):
        """
        Test 'blkid' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.blkid.parse(self.ubuntu_18_4_blkid, quiet=True), self.ubuntu_18_4_blkid_json)

    def test_blkid_sda2_centos_7_7(self):
        """
        Test 'blkid /dev/sda2' on Centos 7.7
        """
        self.assertEqual(jc.parsers.blkid.parse(self.centos_7_7_blkid_sda2, quiet=True), self.centos_7_7_blkid_sda2_json)

    def test_blkid_sda2_ubuntu_18_4(self):
        """
        Test 'blkid /dev/sda2' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.blkid.parse(self.ubuntu_18_4_blkid_sda2, quiet=True), self.ubuntu_18_4_blkid_sda2_json)

    def test_blkid_ip_udev_centos_7_7(self):
        """
        Test 'blkid -ip -o udev /dev/sda2' on Centos 7.7
        """
        self.assertEqual(jc.parsers.blkid.parse(self.centos_7_7_blkid_ip_udev, quiet=True), self.centos_7_7_blkid_ip_udev_json)

    def test_blkid_ip_udev_ubuntu_18_4(self):
        """
        Test 'blkid -ip -o udev /dev/sda2' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.blkid.parse(self.ubuntu_18_4_blkid_sda2, quiet=True), self.ubuntu_18_4_blkid_sda2_json)

    def test_blkid_ip_multi_centos_7_7(self):
        """
        Test 'blkid -ip /dev/sda1 /dev/sda2' on Centos 7.7
        """
        self.assertEqual(jc.parsers.blkid.parse(self.centos_7_7_blkid_ip_multi, quiet=True), self.centos_7_7_blkid_ip_multi_json)

    def test_blkid_ip_multi_ubuntu_18_4(self):
        """
        Test 'blkid -ip /dev/sda1 /dev/sda2' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.blkid.parse(self.ubuntu_18_4_blkid_ip_multi, quiet=True), self.ubuntu_18_4_blkid_ip_multi_json)

    def test_blkid_ip_udev_multi_centos_7_7(self):
        """
        Test 'blkid -ip -o udev /dev/sda1 /dev/sda2' on Centos 7.7
        """
        self.assertEqual(jc.parsers.blkid.parse(self.centos_7_7_blkid_ip_udev_multi, quiet=True), self.centos_7_7_blkid_ip_udev_multi_json)

    def test_blkid_ip_udev_multi_ubuntu_18_4(self):
        """
        Test 'blkid -ip -o udev /dev/sda1 /dev/sda2' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.blkid.parse(self.ubuntu_18_4_blkid_ip_udev_multi, quiet=True), self.ubuntu_18_4_blkid_ip_udev_multi_json)


if __name__ == '__main__':
    unittest.main()



r"""jc - JSON Convert `/etc/os-release` file parser

This parser is an alias to the Key/Value parser (`--kv`).

Usage (cli):

    $ cat /etc/os-release | jc --os-release

Usage (module):

    import jc
    result = jc.parse('os_release', os_release_output)

Schema:

    {
        "<key>":     string
    }

Examples:

    $ cat /etc/os-release | jc --os-release -p
    {
      "NAME": "CentOS Linux",
      "VERSION": "7 (Core)",
      "ID": "centos",
      "ID_LIKE": "rhel fedora",
      "VERSION_ID": "7",
      "PRETTY_NAME": "CentOS Linux 7 (Core)",
      "ANSI_COLOR": "0;31",
      "CPE_NAME": "cpe:/o:centos:centos:7",
      "HOME_URL": "https://www.centos.org/",
      "BUG_REPORT_URL": "https://bugs.centos.org/",
      "CENTOS_MANTISBT_PROJECT": "CentOS-7",
      "CENTOS_MANTISBT_PROJECT_VERSION": "7",
      "REDHAT_SUPPORT_PRODUCT": "centos",
      "REDHAT_SUPPORT_PRODUCT_VERSION": "7"
    }

    $ cat /etc/os-release | jc --os-release -p -r
    {
      "NAME": "\\"CentOS Linux\\"",
      "VERSION": "\\"7 (Core)\\"",
      "ID": "\\"centos\\"",
      "ID_LIKE": "\\"rhel fedora\\"",
      "VERSION_ID": "\\"7\\"",
      "PRETTY_NAME": "\\"CentOS Linux 7 (Core)\\"",
      "ANSI_COLOR": "\\"0;31\\"",
      "CPE_NAME": "\\"cpe:/o:centos:centos:7\\"",
      "HOME_URL": "\\"https://www.centos.org/\\"",
      "BUG_REPORT_URL": "\\"https://bugs.centos.org/\\"",
      "CENTOS_MANTISBT_PROJECT": "\\"CentOS-7\\"",
      "CENTOS_MANTISBT_PROJECT_VERSION": "\\"7\\"",
      "REDHAT_SUPPORT_PRODUCT": "\\"centos\\"",
      "REDHAT_SUPPORT_PRODUCT_VERSION": "\\"7\\""
    }
"""
from jc.jc_types import JSONDictType
import jc.parsers.ini as ini


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.2'
    description = '`/etc/os-release` file parser'
    author = 'Kelly Brazil'
    author_email = 'kellyjonbrazil@gmail.com'
    details = 'Using the ini parser'
    compatible = ['linux', 'darwin', 'cygwin', 'win32', 'aix', 'freebsd']
    tags = ['file', 'standard', 'string']


__version__ = info.version


def parse(
    data: str,
    raw: bool = False,
    quiet: bool = False
) -> JSONDictType:
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        Dictionary. Raw or processed structured data.
    """
    # This parser is an alias of ini.py
    ini.info = info  # type: ignore
    ini.__name__ = __name__
    return ini.parse(data, raw, quiet)



r"""jc - JSON Convert `net localgroup` command output parser

Usage (cli):

    $ net localgroup | jc --net-localgroup
    $ net localgroup /domain | jc --net-localgroup
    $ net localgroup Administrators | jc --net-localgroup
    $ net localgroup Administrators /domain | jc --net-localgroup

Usage (module):

    import jc
    result = jc.parse('net_localgroup', net_localgroup_command_output)

Schema:

    {
        "account_origin":     string,
        "domain":             string,
        "comment":            string,
        "groups": [
            {
                "name":       string
                "members": [
                              string
                ]
            }
        ],
    }

Examples:

    $ net localgroup | jc --net-localgroup -p
    {
        "account_origin": null,
        "comment": null,
        "domain": null,
        "groups": [
            {
                "name": "Administrators",
                "members": [
                    "Administrator",
                    "Operator",
                    "ansible",
                    "user1"
                ]
            }
        ]
    }
"""
import re
import jc.utils


class info():
    """Provides parser metadata (version, author, etc.)"""
    version = '1.0'
    description = '`net localgroup` command parser'
    author = 'joehacksalot'
    author_email = 'joehacksalot@gmail.com'
    compatible = ['win32']
    magic_commands = ['net localgroup']
    tags = ['command']


__version__ = info.version


def parse(data, raw=False, quiet=False):
    """
    Main text parsing function

    Parameters:

        data:        (string)  text data to parse
        raw:         (boolean) unprocessed output if True
        quiet:       (boolean) suppress warning messages if True

    Returns:

        Parsed dictionary. The raw and processed data structures are the same.
    """
    jc.utils.compatibility(__name__, info.compatible, quiet)
    jc.utils.input_type_check(data)

    raw_output = {}
    if jc.utils.has_data(data):
        # Initialize the parsed output dictionary with all fields set to None or empty lists
        raw_output = _parse(data)

    return raw_output if raw else _process(raw_output)

def _process(proc_data):
    """
    Final processing to conform to the schema.

    Parameters:

        proc_data:   (Dictionary) raw structured data to process

    Returns:

        Processed Dictionary. Structured data to conform to the schema.
    """
    return proc_data


class _PushbackIterator:
    def __init__(self, iterator):
        self.iterator = iterator
        self.pushback_stack = []
    def __iter__(self):
        return self
    def __next__(self):
        if self.pushback_stack:
            return self.pushback_stack.pop()
        else:
            return next(self.iterator)
    def pushback(self, value):
        self.pushback_stack.append(value)

def _parse_net_localgroup_list(line_iter, expect_asterisk):
    name_list = []
    while True:
        try:
            line = next(line_iter)
            if not line.strip():
                continue  # Skip empty lines

            # Check if the line starts with an asterisk
            if line == 'The command completed successfully.':
                break
            elif expect_asterisk and line.startswith('*'):
                name_list.append(line[1:].strip())
            else:
                name_list.append(line)
        except StopIteration:
            break
    return name_list


def _parse(data):
    lines = data.splitlines()
    parse_type = None  # Can be 'groups_list' or 'members'

    result = {
        "account_origin": None,
        "domain": None,
        "comment": None,
        "groups": []
    }

    group_name = ""
    lines = data.splitlines()
    lines = [line.rstrip() for line in lines if line.strip() != ""]

    line_iter = _PushbackIterator(iter(lines))
    for line in line_iter:
        line = line.rstrip()

        # Skip empty lines
        if not line.strip():
            continue

        match_domain_processed = re.match(r"^The request will be processed at a domain controller for domain (.+)", line, re.IGNORECASE)
        match_localgroup_list = re.match(r"^Aliases for[\s]*([^:]+)", line, re.IGNORECASE)        #  "Aliases for \\DESKTOP-WIN11:"
        match_localgroup_members = re.match(r"^Alias name[\s]*([^:]+)", line, re.IGNORECASE)      #  "Alias name     administrators:"
        if match_domain_processed:
            # Extract the domain name
            result["domain"] = match_domain_processed.group(1).strip()
        elif match_localgroup_list:
            # Extract the account origin
            result["account_origin"] = match_localgroup_list.group(1).strip()
            parse_type = 'groups_list'   # Prepare to read groups
        elif match_localgroup_members:
            # We are querying a specific group
            group_name = match_localgroup_members.group(1).strip()
            parse_type = 'members_list'  # Prepare to read members
        elif line.startswith('Comment'):
            comment_line = line.split('Comment', 1)[1].strip()
            result["comment"] = comment_line if comment_line else None
        elif line.startswith('---'):
            # Start of a section (groups or members)
            if parse_type == 'groups_list':
                names_list = _parse_net_localgroup_list(line_iter, expect_asterisk=True)
                result["groups"] = [{"name": group_name, "members": []} for group_name in names_list]
            elif parse_type == 'members_list':
                names_list = _parse_net_localgroup_list(line_iter, expect_asterisk=False)
                result["groups"] = [{
                    "name": group_name,
                    "members": names_list
                }]

    return result


import os
import unittest
import json
import jc.parsers.ntpq

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):

    # input
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/ntpq-p.out'), 'r', encoding='utf-8') as f:
        centos_7_7_ntpq_p = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ntpq-p.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_ntpq_p = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/ntpq-pn.out'), 'r', encoding='utf-8') as f:
        centos_7_7_ntpq_pn = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ntpq-pn.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_ntpq_pn = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ntpq-p2.out'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_ntpq_p2 = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/freebsd12/ntpq-p.out'), 'r', encoding='utf-8') as f:
        freebsd12_ntpq_p = f.read()

    # output
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/ntpq-p.json'), 'r', encoding='utf-8') as f:
        centos_7_7_ntpq_p_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ntpq-p.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_ntpq_p_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/ntpq-pn.json'), 'r', encoding='utf-8') as f:
        centos_7_7_ntpq_pn_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ntpq-pn.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_ntpq_pn_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ntpq-p2.json'), 'r', encoding='utf-8') as f:
        ubuntu_18_4_ntpq_p2_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/freebsd12/ntpq-p.json'), 'r', encoding='utf-8') as f:
        freebsd12_ntpq_p_json = json.loads(f.read())


    def test_ntpq_p_nodata(self):
        """
        Test 'ntpq -p' with no data
        """
        self.assertEqual(jc.parsers.ntpq.parse('', quiet=True), [])

    def test_ntpq_p_centos_7_7(self):
        """
        Test 'ntpq -p' on Centos 7.7
        """
        self.assertEqual(jc.parsers.ntpq.parse(self.centos_7_7_ntpq_p, quiet=True), self.centos_7_7_ntpq_p_json)

    def test_ntpq_p_ubuntu_18_4(self):
        """
        Test 'ntpq -p' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.ntpq.parse(self.ubuntu_18_4_ntpq_p, quiet=True), self.ubuntu_18_4_ntpq_p_json)

    def test_ntpq_pn_centos_7_7(self):
        """
        Test 'ntpq -pn' on Centos 7.7
        """
        self.assertEqual(jc.parsers.ntpq.parse(self.centos_7_7_ntpq_pn, quiet=True), self.centos_7_7_ntpq_pn_json)

    def test_ntpq_pn_ubuntu_18_4(self):
        """
        Test 'ntpq -pn' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.ntpq.parse(self.ubuntu_18_4_ntpq_pn, quiet=True), self.ubuntu_18_4_ntpq_pn_json)

    def test_ntpq_p2_ubuntu_18_4(self):
        """
        Test 'ntpq -p' with ip data with spaces on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.ntpq.parse(self.ubuntu_18_4_ntpq_p2, quiet=True), self.ubuntu_18_4_ntpq_p2_json)

    def test_ntpq_p_freebsd12(self):
        """
        Test 'ntpq -p' on FreeBSD12
        """
        self.assertEqual(jc.parsers.ntpq.parse(self.freebsd12_ntpq_p, quiet=True), self.freebsd12_ntpq_p_json)


if __name__ == '__main__':
    unittest.main()



import os
import json
import unittest
import jc.parsers.ip_route

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


class MyTests(unittest.TestCase):

    # INPUT
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ip_route.out'), 'r',
              encoding='utf-8') as f:
        ubuntu_18_4_ip_route = f.read()

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/ip_route.out'), 'r',
              encoding='utf-8') as f:
        centos_7_7_ip_route = f.read()

    # OUTPUT
    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/ubuntu-18.04/ip_route.json'), 'r',
              encoding='utf-8') as f:
        ubuntu_18_4_ip_route_json = json.loads(f.read())

    with open(os.path.join(THIS_DIR, os.pardir, 'tests/fixtures/centos-7.7/ip_route.json'), 'r',
              encoding='utf-8') as f:
        centos_7_7_ip_route_json = json.loads(f.read())

    def test_ip_route_nodata(self):
        """
        Test  'ip_route' with no data
        """
        self.assertEqual(jc.parsers.ip_route.parse('', quiet=True), [])

    def test_ip_route_ubuntu_18_4(self):
        """
        Test 'ip_route' on Ubuntu 18.4
        """
        self.assertEqual(jc.parsers.ip_route.parse(self.ubuntu_18_4_ip_route, quiet=True),
                         self.ubuntu_18_4_ip_route_json)

    def test_ip_route_centos_7_7(self):
        """
        Test 'history' on Centos 7.7
        """
        self.assertEqual(jc.parsers.ip_route.parse(self.centos_7_7_ip_route, quiet=True),
                         self.centos_7_7_ip_route_json)


if __name__ == '__main__':
    unittest.main()
